@@ -110,14 +110,16 @@ class AppletOrientationHelper:
     def __init__(self, widget, applet, track_widget):
         update_cb = lambda: self.update_position(widget, track_widget, applet)
 
-        widget.connect("realize", lambda w: update_cb())
+        widget.connect("size-allocate", lambda w, a: update_cb())
         track_widget.connect("size-allocate", lambda w, a: update_cb())
+        track_widget.connect("screen-changed",
+                             lambda w, screen: widget.set_screen(screen))
 
     def update_position(self, widget, track_widget, applet):
         '''
         Calculates position and moves window to it.
         '''
-        if not track_widget.flags() & gtk.REALIZED or not widget.flags() & gtk.REALIZED:
+        if not track_widget.flags() & gtk.REALIZED:
             return
 
         # Get our own dimensions & position
@@ -125,9 +127,13 @@ class AppletOrientationHelper:
         track_x += track_widget.allocation.x
         track_y += track_widget.allocation.y
 
-        width, height = widget.size_request()
+        if widget.flags() & gtk.REALIZED:
+            width = widget.allocation.width
+            height = widget.allocation.height
+        else:
+            width, height = widget.size_request()
 
-        screen = widget.get_screen()
+        screen = track_widget.get_screen()
         monitor = screen.get_monitor_geometry(screen.get_monitor_at_window(track_widget.window))
 
         orient = applet.get_orient()
@@ -168,8 +174,9 @@ class AppletOrientationHelper:
             if x < 0:
                 x = 0
 
+        if (gravity != widget.get_gravity()):
+            widget.set_gravity(gravity)
         widget.move(x, y)
-        widget.set_gravity(gravity)
 
 
 class TopicMenu(GrabOnShowWindow, TopicView):
@@ -180,7 +187,7 @@ class TopicMenu(GrabOnShowWindow, TopicView):
         GrabOnShowWindow.__init__(self)
         TopicView.__init__(self, topic)
 
-        self.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_MENU)
+        self.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_POPUP_MENU)
         self.set_decorated(False)
 
         self.add_accel_group(self.accel_group)
@@ -214,22 +221,25 @@ class TopicMenu(GrabOnShowWindow, TopicView):
         ev_box.add(frame)
 
         # Contains the visual frame, giving it some space
-        content = gtk.VBox(False, 0)
-        content.set_border_width(0)
-        content.show()
-        frame.add(content)
+        self._content = gtk.VBox(False, 0)
+        self._content.set_border_width(0)
+        self._content.show()
+        frame.add(self._content)
+
+        # Toolbar
+        self._content.pack_start(self.toolbar, False, False, 0)
 
         # Hbox containing the sidebar buttons and the toolbar/iconview
         body = gtk.HBox(False, 12)
         body.set_border_width(12)
         body.show()
-        content.pack_start(body, True, True, 0)
-
-        # Toolbar
-        content.pack_start(self.toolbar, False, False, 0)
+        self._content.pack_start(body, True, True, 0)
 
         # Load up the sidebar
-        body.pack_start(self.sidebar, False, False, 0)
+        self._sidebar_align = gtk.Alignment(0.5, 0.0, 0.0, 0.0)
+        self._sidebar_align.add(self.sidebar)
+        self._sidebar_align.show()
+        body.pack_start(self._sidebar_align, False, True, 0)
 
         # Horizontal time line bar with plus/minus buttons above icon view
         self.timebar = TimeBar()
@@ -239,6 +249,11 @@ class TopicMenu(GrabOnShowWindow, TopicView):
         # Frame containing timebar and iconview inside scrolled window
         body.pack_start(self.content_frame, True, True, 0)
 
+        # Bound the height request of the scrolled window and icon view
+        self.scroll.connect("size-request", self._scroll_get_best_size)
+        self.scroll.get_vadjustment().connect("changed",
+                                              lambda adj: self.scroll.queue_resize_no_redraw())
+
         self.view.set_size_request(500, -1)
         self.view.connect("item-activated", lambda v, p: self.hide()) # Iconify on item open
 
@@ -248,6 +263,24 @@ class TopicMenu(GrabOnShowWindow, TopicView):
         # Select an initial sidebar button
         self.find_first_button()
 
+    def set_gravity(self, gravity):
+        # Override GtkWindow::set_gravity
+        assert self.toolbar.parent == self._content
+
+        self._content.remove(self.toolbar)
+        if gravity in (gtk.gdk.GRAVITY_SOUTH,
+                       gtk.gdk.GRAVITY_SOUTH_WEST,
+                       gtk.gdk.GRAVITY_SOUTH_EAST):
+            # Toolbar & sidebar at the bottom
+            self._content.pack_end(self.toolbar, False, False, 0)
+            self._sidebar_align.set_property("yalign", 1.0)
+        else:
+            # Toolbar & sidebar at the top
+            self._content.pack_start(self.toolbar, False, False, 0)
+            self._sidebar_align.set_property("yalign", 0.0)
+
+        gtk.Window.set_gravity(self, gravity)
+
     def get_zoom_level(self):
         return self.timebar.get_zoom_level()
 
@@ -265,6 +298,43 @@ class TopicMenu(GrabOnShowWindow, TopicView):
         else:
             self.timebar.hide()
 
+    def _scroll_get_best_size(self, scroll, req):
+        '''
+        Sets the height request of the scrolled window to either the upper bound
+        of the scrollbar or 70% of the screen height, whichever is smaller.
+        '''
+        vadj = scroll.get_property("vadjustment")
+        upper = int(vadj.get_property("upper"))
+        upper = min(upper, int(self.get_screen().get_height() * 0.7))
+
+        if upper == 0 or upper == scroll.allocation.height:
+            # Avoid unneeded or bogus resize
+            return False
+
+        req.height = upper
+        return True
+
+    def _load_items_done(self, ondone_cb):
+        '''
+        Do some magic to get a decent size request out of an iconview.  Reset
+        the scrolled window upper bound to 0, and resize the window.
+        '''
+        if ondone_cb:
+            ondone_cb()
+
+        vadj = self.scroll.get_property("vadjustment")
+        vadj.set_property("upper", 0)
+
+        w, h = self.child.size_request()
+        self.resize(w, h)
+
+    def load_items(self, items, ondone_cb = None):
+        '''
+        Wrap the ondone callback to resize the scrolled window after reloading
+        the iconview.
+        '''
+        self.view.load_items(items, lambda: self._load_items_done(ondone_cb))
+
     def _favorite_selection_changed(self, view, fav, block_id, img):
         # Block the toggled handler from triggering
         fav.handler_block(block_id)
@@ -272,14 +342,14 @@ class TopicMenu(GrabOnShowWindow, TopicView):
         if len(selected) == 1:
             model = view.get_model()
             item = model.get_value(model.get_iter(selected[0]), 2)
-            fav.set_active(item.get_is_pinned())
 
             pix = icon_factory.load_icon("gnome-favorites", gtk.ICON_SIZE_LARGE_TOOLBAR)
             if not item.get_is_pinned():
                pix = icon_factory.greyscale(pix)
             img.set_from_pixbuf(pix)
 
-            fav.set_sensitive(True)
+            fav.set_active(item.get_is_pinned())
+            fav.set_sensitive(item.get_can_pin())
         else:
             fav.set_active(False)
             fav.set_sensitive(False)
@@ -323,21 +393,21 @@ class TopicMenu(GrabOnShowWindow, TopicView):
         return sep
 
     def _add_toolbar_items(self):
+        # Right-align the zoom and search tool items
+        self.toolbar.insert(self._make_toolbar_expander(), -1)
+
         for i in self.topic.get_toolbar_items(self.tooltips):
             ### Uncomment to pack space between elements
             #self.toolbar.insert(self._make_toolbar_expander(), -1)
             if i:
                 i.show_all()
                 if isinstance(i, ToolMenuButton):
-                    for menu_item in i.get_menu():
+                    for menu_item in i.get_menu() or []:
                         menu_item.connect("activate", lambda menu_item: self.hide())
                 elif isinstance(i, gtk.ToolButton):
                     i.connect("clicked", lambda btn: self.hide())
                 self.toolbar.insert(i, -1)
 
-        # Right-align the zoom and search tool items
-        self.toolbar.insert(self._make_toolbar_expander(), -1)
-
         self.add_favorite_toolitem()
 
         # Try to disassociate the Favorite button from the search box
@@ -345,6 +415,9 @@ class TopicMenu(GrabOnShowWindow, TopicView):
 
         self.add_search_toolitem()
 
+        # Right-align the zoom and search tool items
+        self.toolbar.insert(self._make_toolbar_expander(), -1)
+
 
 def get_topic_window_mod(self):
     if not self.topic_window:
@@ -534,7 +607,9 @@ class GimmieApplet:
             btn.show()
             self.content.pack_start(btn, True, True, 0)
 
-            self.prefs.connect("changed::" + pref_key, self._show_topic_pref_changed, btn, pref_key)
+            self.prefs.connect("changed::" + pref_key,
+                               self._show_topic_pref_changed, btn,
+                               pref_key)
             self._show_topic_pref_changed(self.prefs, btn, pref_key)
 
         self.prefs.connect("changed::use_colors", self._use_colors_pref_changed)
@@ -545,7 +620,8 @@ class GimmieApplet:
         about = gtk.AboutDialog()
         about.set_name("Gimmie")
         about.set_authors(("Alex Graveley <alex@beatniksoftware.com>",
-                           "David Trowbridge <trowbrds@gmail.com>"))
+                           "David Trowbridge <trowbrds@gmail.com>",
+                           "Tony Tsui <tsui.tony@gmail.com"))
         about.run()
         about.destroy()
         return
@@ -572,7 +648,10 @@ def gimmie_run_standalone():
     main_window.connect("destroy", gtk.main_quit)
 
     app = gnomeapplet.Applet()
+
+    # NOTE: Change this to test different panel orientations
     app.get_orient = lambda: gnomeapplet.ORIENT_DOWN
+    
     gimmie_applet_factory(app, None)
     app.reparent(main_window)
 
@@ -601,6 +680,9 @@ def main(args):
         Preferences(GConfBridge()).run()
         sys.exit()
 
+    # Tell gobject/gtk we are threaded
+    gtk.gdk.threads_init()
+
     gnomeapplet.bonobo_factory("OAFIID:GNOME_Gimmie_Factory", 
                                gnomeapplet.Applet.__gtype__, 
                                "gimmie", "0", gimmie_applet_factory)
