@@ -16,10 +16,10 @@ except ImportError:
 import gdmclient
 
 from gimmie_globals import gimmie_is_panel_applet
-from gimmie_base import Item, ItemSource, DisabledItemSource, Topic, IOrientationAware
+from gimmie_base import Item, ItemSource, Topic, IOrientationAware, gimmie_get_topic_for_uri
 from gimmie_file import FileItem
 from gimmie_logout import LogoutDialog
-from gimmie_recent import RecentlyUsedOfMimeType
+from gimmie_recent import RecentAggregate, recent_model
 from gimmie_util import *
 
 # FIXME: Move these to another file?
@@ -27,71 +27,30 @@ from gimmie_applications import MenuTree, RecentSettingsLaunchers, LauncherItem
 from gimmie_trash import TrashItem
 
 
-class RecentComputerItems(RecentlyUsedOfMimeType):
-    '''
-    This RecentlyUsedOfMimeType subclass maps recently used Items onto Items
-    with matching URIs from a list of other ItemSources.
-    '''
-    default_mime_type_filter = ("x-directory/normal",
-                                "x-gimmie/printer",
-                                "application/octet-stream")
-
-    def __init__(self,
-                 sources,
-                 mimetype_list = default_mime_type_filter):
-        RecentlyUsedOfMimeType.__init__(self,
-                                        name=_("Recently Used"),
-                                        icon="stock_calendar",
-                                        mimetype_list=mimetype_list)
-
-        # Sources provide the real items we will display
-        self.sources = sources
-        for source in sources:
-            self._listen_to_source(source)
-
-        # Throw settings launchers in too
-        self.settings_launchers = RecentSettingsLaunchers(None, None)
-        self._listen_to_source(self.settings_launchers)
-
-    def _listen_to_source(self, source):
-        source.connect_after("reload", lambda x: self.emit("reload"))
-
-    def do_reload(self):
-        RecentlyUsedOfMimeType.do_reload(self) # Reload our items
-        items = []
-
-        # Include recent settings
-        items += self.settings_launchers.get_items()
-
-        # Find items matching recent uris
-        recent_uris = [x.get_uri() for x in self.get_items()]
-        for source in self.sources:
-            items += [x for x in source.get_items() if x.get_uri() in recent_uris]
-
-        self.set_items(items) # Override our items
-
+#
+# Sidebar ItemSources
+#
 
 class FavoritesSource(ItemSource):
     '''
     Item source that lists all favorite items.
     '''
-    def __init__(self, items = None, uri = None):
+    def __init__(self):
         ItemSource.__init__(self,
                             name=_("All Favorites"),
                             icon="gnome-favorites",
-                            uri=uri,
+                            uri="source://AllFavorites",
                             filter_by_date=False)
         bookmarks.connect("reload", lambda x: self.emit("reload"))
 
-    def do_reload(self):
-        items = []
+    def get_items_uncached(self):
         for uri, itemclass in bookmarks.get_bookmarks_and_class():
             try:
                 mod, cls = itemclass.rsplit(".", 1)
                 dynmod = __import__(mod, None, None, cls)
                 dyncls = getattr(dynmod, cls)
                 dynobj = dyncls(uri=uri)
-                items.append(dynobj)
+                yield dynobj
             except (ValueError, TypeError, ImportError, AttributeError), err:
                 # ValueError - thrown by Item constructor, or strange itemclass
                 # TypeError - thrown by Item not accepting uri arg
@@ -99,8 +58,6 @@ class FavoritesSource(ItemSource):
                 # AttributeError - mod doesn't contain cls
                 print "Error creating %s for URI \"%s\": %s" % (itemclass, uri, err)
 
-        self.set_items(items)
-
 
 class PlacesSource(ItemSource):
     '''
@@ -108,26 +65,49 @@ class PlacesSource(ItemSource):
     '''
     
     def __init__(self):
-        ItemSource.__init__(self, _("Places"), icon="gnome-fs-directory", filter_by_date=False)
+        ItemSource.__init__(self,
+                            name=_("Places"),
+                            icon="gnome-fs-directory",
+                            uri="source://Places",
+                            filter_by_date=False)
 
         # Listen for changes in the ~/.gtk-bookmarks file
         places.connect("reload", lambda x: self.emit("reload"))
 
-    def do_reload(self):
-        items = []
+    def get_items_uncached(self):
         for uri, name, mime, icon in places.get_places():
-            items.append(FileItem(uri=uri, icon=icon))
+            yield FileItem(uri=uri, icon=icon)
 
         # Add a trashcan so people can toggle it's pinned setting on the gimmie bar.
-        items.append(TrashItem())
-
-        self.set_items(items)
+        yield TrashItem()
 
 
 class DriveItem(Item):
-    def __init__(self, drive):
+    def __init__(self, drive = None, uri = None):
+        assert drive or uri, "Invalid arguments passed to DriveItem.__init__"
+
+        if not drive and uri:
+            for drive in gnomevfs.VolumeMonitor().get_connected_drives():
+                if drive.get_activation_uri() == uri:
+                    break
+            else:
+                raise ValueError, "Cannot find drive to match URI '%s'" % uri
+
+        uri = drive.get_activation_uri()
+        if not uri:
+            for volume in drive.get_mounted_volumes():
+                if volume.is_user_visible():
+                    # FIXME: Using the first volume URI for a device is
+                    #        broken. There could be multiple, though I don't
+                    #        know under what circumstances this would happen.
+                    uri = volume.get_activation_uri()
+                    break
+            else:
+                raise ValueError, "Cannot find URI to open for drive '%s'" % \
+                      drive.get_display_name()
+
         Item.__init__(self,
-                      uri=drive.get_activation_uri(),
+                      uri=uri,
                       timestamp=0,
                       mimetype="x-gimmie/drive",
                       icon=drive.get_icon())
@@ -152,7 +132,7 @@ class DriveItem(Item):
             # FIXME: If read-only drive, show allocated size instead
             try:
                 space = gnomevfs.get_volume_free_space(gnomevfs.URI(self.get_uri()))
-            except (TypeError, gnomevfs.NotSupportedError):
+            except (TypeError, gnomevfs.Error):
                 # When the URI or free space is unknown
                 space = 0
 
@@ -176,15 +156,23 @@ class DriveItem(Item):
     def get_is_mounted(self):
         return self.drive.is_mounted()
 
+    def eject(self):
+        try:
+            self.drive.eject(lambda x, y, z: self.emit("reload"))
+        except SystemError, err:
+            # FIXME: Some PyGtk versions are broken.  See bug #418667.
+            print " !!! Error ejecting:", err
+
     def populate_popup(self, menu):
         Item.populate_popup(self, menu)
 
         # FIXME: Add mount/unmount toggle?  Need to track nautilus.
 
         eject = gtk.MenuItem(_("_Eject"), use_underline=True)
-        ### FIXME: PyGTK Bindings not bound yet.
-        #eject.set_sensitive(self.drive.needs_eject())
-        eject.connect("activate", lambda w: self.drive.eject(lambda x, y, z: None))
+        if hasattr(self.drive, "needs_eject"):
+            # NOTE: PyGTK doesn't bind this yet.  See bug #419225.
+            eject.set_sensitive(self.drive.needs_eject())
+        eject.connect("activate", lambda w: self.eject())
         eject.show()
         menu.append(eject)
 
@@ -199,6 +187,7 @@ class DevicesSource(ItemSource):
                             icon="gnome-dev-removable-usb",
                             uri="source://Devices",
                             filter_by_date=False)
+        
         self.add_bluetooth = Item(name=_("Add Bluetooth..."),
                                   comment=_("Access a wireless device"),
                                   icon="stock_bluetooth",
@@ -209,7 +198,8 @@ class DevicesSource(ItemSource):
                               comment=_("Burn files onto a CD or DVD disk"),
                               icon="gnome-dev-cdrom",
                               special=True)
-        self.cd_burner.do_open = lambda: launcher.launch_command("nautilus --no-desktop burn:///")
+        self.cd_burner.do_open = \
+            lambda: launcher.launch_command("nautilus --no-desktop burn:///")
 
         self.vol_monitor = gnomevfs.VolumeMonitor()
         self.vol_monitor.connect("drive_connected", lambda v, d: self.emit("reload"))
@@ -220,13 +210,15 @@ class DevicesSource(ItemSource):
     def _add_bluetooth(self):
         ImplementMe()
 
-    def do_reload(self):
-        items = []
-        items.append(self.add_bluetooth)
-        items.append(self.cd_burner)
+    def get_items_uncached(self):
+        yield self.add_bluetooth
+        yield self.cd_burner
+
         for drive in self.vol_monitor.get_connected_drives():
-            items.append(DriveItem(drive))
-        self.set_items(items)
+            try:
+                yield DriveItem(drive)
+            except ValueError:
+                pass
 
 
 class PrinterItem(Item):
@@ -236,6 +228,7 @@ class PrinterItem(Item):
     def __init__(self, printer):
         Item.__init__(self, mimetype="x-gimmie/printer")
         self.printer = printer
+        self.add_tag("Printer")
 
         printer.connect("is_default_changed", lambda p: self.emit("reload"))
         printer.connect("attributes_changed", lambda p: self.emit("reload"))
@@ -285,10 +278,7 @@ class PrinterItem(Item):
 
     def do_open(self):
         launcher.launch_command("gnome-cups-manager --view \"%s\"" % self.printer.get_name())
-        
-        recent_item = launcher.make_recent_item(self.get_uri(), "x-gimmie/printer")
-        recent_item.add_group("Printer")
-        recent_model.add_full(recent_item)
+        recent_model.add_item(self)
 
     def _open_properties(self):
         launcher.launch_command("gnome-cups-manager --properties \"%s\"" % self.printer.get_name())
@@ -329,7 +319,7 @@ class PrinterSource(ItemSource):
     '''
     def __init__(self):
         ItemSource.__init__(self,
-                            _("Printers"),
+                            name=_("Printers"),
                             icon=gtk.STOCK_PRINT,
                             uri="source://Printers",
                             filter_by_date=False)
@@ -340,17 +330,20 @@ class PrinterSource(ItemSource):
                                 special=True)
         self.add_printer.do_open = lambda: launcher.launch_command("gnome-cups-add")
 
-        gnomecups.new_printer_notify_add(lambda name: self.emit("reload"))
-        gnomecups.printer_removed_notify_add(lambda name: self.emit("reload"))
+        if gnomecups:
+            gnomecups.new_printer_notify_add(lambda name: self.emit("reload"))
+            gnomecups.printer_removed_notify_add(lambda name: self.emit("reload"))
 
-    def do_reload(self):
-        items = [self.add_printer]
+    def get_enabled(self):
+        return gnomecups != None
 
-        for printer_name in gnomecups.get_printers():
-            printer = gnomecups.printer_get(printer_name)
-            items.append(PrinterItem(printer))
+    def get_items_uncached(self):
+        yield self.add_printer
 
-        self.set_items(items)
+        if gnomecups:
+            for printer_name in gnomecups.get_printers():
+                printer = gnomecups.printer_get(printer_name)
+                yield PrinterItem(printer)
 
 
 class RemoteMountItem(Item):
@@ -358,6 +351,7 @@ class RemoteMountItem(Item):
         Item.__init__(self,
                       name=volume.get_display_name(),
                       uri=volume.get_activation_uri(),
+                      mimetype="x-directory/normal",
                       icon=volume.get_icon())
 
     def get_timestamp(self):
@@ -374,9 +368,7 @@ class RemoteMountItem(Item):
 
     def do_open(self):
         launcher.launch_command("nautilus %s" % self.get_uri())
-
-        recent_item = launcher.make_recent_item(self.get_uri(), "application/octet-stream")
-        recent_model.add_full(recent_item)
+        recent_model.add_item(self)
 
 
 class BonjourSource(ItemSource):
@@ -410,21 +402,297 @@ class BonjourSource(ItemSource):
         self.vol_monitor.connect("volume_mounted", lambda v, d: self.emit("reload"))
         self.vol_monitor.connect("volume_unmounted", lambda v, d: self.emit("reload"))
 
-    def do_reload(self):
-        items = [self.connect_to, self.smb]
-
-        for vol in self.vol_monitor.get_mounted_volumes():
-            if vol.get_volume_type() == gnomevfs.VOLUME_TYPE_CONNECTED_SERVER:
-                items.append(RemoteMountItem(vol))
+    def get_items_uncached(self):
+        yield self.connect_to
+        yield self.smb
 
         # FIXME: List avahi-discovered hosts
 
-        self.set_items(items)
+        for vol in self.vol_monitor.get_mounted_volumes():
+            if vol.get_volume_type() == gnomevfs.VOLUME_TYPE_CONNECTED_SERVER:
+                yield RemoteMountItem(vol)
 
     def _connect_to(self):
         launcher.launch_command("nautilus-connect-server")
 
 
+class SettingsSource(ItemSource):
+    def __init__(self):
+        ItemSource.__init__(self,
+                            name=_("Settings"),
+                            uri="source://Settings",
+                            filter_by_date=False)
+
+        self.inline_administration_items = True
+        self.settings_menu_tree = None
+        try:
+            # Gnome 2.17.91+ changed the file name arbitrarily.  Yay.
+            self.settings_menu_tree = MenuTree("gnomecc.menu")
+            # Administration items are not included in the new gnomecc.menu
+            self.inline_administration_items = False
+        except ValueError:
+            try:
+                # This is the file panel menu-based gnomecc uses.  It generally
+                # includes preferences.menu and system-settings.menu.
+                self.settings_menu_tree = MenuTree("settings.menu")
+            except ValueError:
+                try:
+                    # Some distros rename settings.menu.
+                    self.settings_menu_tree = MenuTree("gnome-settings.menu")
+                except ValueError:
+                    pass
+
+        if self.settings_menu_tree:
+            self.settings_menu_tree.connect_after("reload", lambda x: self.emit("reload"))
+
+        self.settings_menu_source = None
+        self.do_reload()
+
+    def do_reload(self):
+        self.settings_menu_source = self.settings_menu_tree.get_toplevel_flat_source()
+
+    def always_show_descriptions(self):
+        return True
+
+    def get_enabled(self):
+        return self.settings_menu_tree != None
+
+    def get_icon(self, size):
+        return self.settings_menu_source.get_icon(size)
+
+    def get_comment(self):
+        return self.settings_menu_source.get_comment()
+
+    def get_items_uncached(self):
+        if self.settings_menu_source:
+            for item in self.settings_menu_source.get_items_uncached():
+                yield item
+
+    def get_settings_menu_tree(self):
+        return self.settings_menu_tree
+
+    def has_administration(self):
+        return self.inline_administration_items
+
+
+class AdministrationSource(ItemSource):
+    def __init__(self):
+        ItemSource.__init__(self,
+                             name=_("Administration"),
+                             uri="source://Administration",
+                             filter_by_date=False)
+
+        self.system_settings_menu_tree = MenuTree("system-settings.menu")
+        self.system_settings_menu_tree.connect("reload", lambda x: self.emit("reload"))
+
+        self.system_settings_menu_source = None
+        self.do_reload()
+
+    def do_reload(self):
+        self.system_settings_menu_source = self.system_settings_menu_tree.get_toplevel_flat_source()
+
+    def always_show_descriptions(self):
+        return True
+
+    def get_enabled(self):
+        return self.system_settings_menu_tree != None
+
+    def get_icon(self, size):
+        return self.system_settings_menu_source.get_icon(size)
+
+    def get_comment(self):
+        return self.system_settings_menu_source.get_comment()
+
+    def get_items_uncached(self):
+        if self.system_settings_menu_source:
+            for item in self.system_settings_menu_source.get_items_uncached():
+                yield item
+
+
+#
+# Toolbar Items
+#
+
+class UserMenu(gtk.Menu):
+    def __init__(self):
+        gtk.Menu.__init__(self)
+
+        btn = gtk.ImageMenuItem(_("About Me"))
+        btn.set_image(icon_factory.load_image("user-info", gtk.ICON_SIZE_LARGE_TOOLBAR))
+        btn.connect("activate", lambda b: launcher.launch_command("gnome-about-me"))
+        btn.show()
+        self.append(btn)
+
+        btn = gtk.ImageMenuItem(_("Switch User"))
+        btn.set_image(icon_factory.load_image("gnome-lockscreen", gtk.ICON_SIZE_LARGE_TOOLBAR))
+        btn.connect("activate", self._switch_user)
+        btn.show()
+        self.append(btn)
+
+        btn = gtk.ImageMenuItem(_("Log Out..."))
+        btn.set_image(icon_factory.load_image("gnome-logout", gtk.ICON_SIZE_LARGE_TOOLBAR))
+        btn.connect("activate", self._logout, LogoutDialog.LOGOUT)
+        btn.show()
+        self.append(btn)
+
+    def _logout(self, btn, method):
+        dialog = LogoutDialog(method)
+        dialog.show()
+
+    def _switch_user(self, button):
+        gdmclient.new_login()
+
+
+class UserToolMenuButton(ToolMenuButton):
+    def __init__(self, tooltips):
+        # Get the user's real name from /etc/passwd
+        pwent = pwd.getpwuid(os.getuid())
+        self.username = pwent.pw_gecos.split(",")[0] or pwent.pw_name
+
+        self.face_monitor = FileMonitor(os.path.expanduser("~/.face"))
+        self.face_monitor.open()
+
+        img = gtk.Image()
+        self.face_monitor.connect("event", lambda m, u, ev, img: self._dot_face_changed, img)
+        self._dot_face_changed(img)
+        
+        ToolMenuButton.__init__(self, img, self.username)
+        self.set_tooltip(tooltips, _("Switch to another user"))
+        self.set_menu(UserMenu())
+
+    def _dot_face_changed(self, img):
+        face = os.path.expanduser("~/.face")
+        if not os.path.exists(face):
+            face = "stock_person"
+        img.set_from_pixbuf(icon_factory.load_icon(face, gtk.ICON_SIZE_LARGE_TOOLBAR))
+
+
+class ShutdownToolButton(gtk.ToolButton):
+    __gsignals__ = {
+        "clicked" : "override"
+        }
+
+    def __init__(self, tooltips):
+        img = icon_factory.load_image("gnome-shutdown", gtk.ICON_SIZE_LARGE_TOOLBAR)
+        gtk.ToolButton.__init__(self, img, _("Shutdown..."))
+        self.set_tooltip(tooltips, _("Shutdown or suspend this computer"))
+
+    def do_clicked(self):
+        dialog = LogoutDialog(LogoutDialog.SHUTDOWN)
+        dialog.show()
+
+
+class HelpToolMenuButton(ToolMenuButton):
+    def __init__(self, tooltips):
+        img = icon_factory.load_image(gtk.STOCK_HELP, gtk.ICON_SIZE_LARGE_TOOLBAR)
+        ToolMenuButton.__init__(self, img, _("Help"))
+
+        menu = gtk.Menu()
+        self.set_menu(menu)
+
+        about = gtk.ImageMenuItem(_("About Gnome"))
+        about.set_image(icon_factory.load_image(gtk.STOCK_ABOUT, gtk.ICON_SIZE_LARGE_TOOLBAR))
+        about.connect("activate", lambda w: launcher.launch_command("gnome-about"))
+        about.show()
+        menu.append(about)
+        
+
+class VolumeToolMenuButton(ToolMenuButton):
+    '''Dummy placeholder until this implements a volume control'''
+    def __init__(self, tooltips):
+        img = icon_factory.load_image("gnome-audio", gtk.ICON_SIZE_LARGE_TOOLBAR)
+        ToolMenuButton.__init__(self, img, "")
+        self.set_tooltip(tooltips, _("Volume: 75%"))
+
+        menu = gtk.Menu()
+        self.set_menu(menu)
+
+        mi = gtk.MenuItem(_("Not Implemented"))
+        mi.set_sensitive(False)
+        mi.show()
+        menu.append(mi)
+
+
+class NetworkLocationToolMenuButton(ToolMenuButton):
+    '''Dummy placeholder until this implements NetworkManager-applet behavior'''
+    def __init__(self, tooltips):
+        img = icon_factory.load_image(gtk.STOCK_NETWORK, gtk.ICON_SIZE_LARGE_TOOLBAR)
+        ToolMenuButton.__init__(self, img, "")
+        self.set_tooltip(tooltips, _("Change the network location"))
+
+        menu = gtk.Menu()
+        self.set_menu(menu)
+
+        mi = gtk.MenuItem(_("Not Implemented"))
+        mi.set_sensitive(False)
+        mi.show()
+        menu.append(mi)
+
+
+class SettingsMenu(gtk.Menu):
+    '''
+    Lists the Settings items split into columns by toplevel categorization,
+    i.e. Preferences and Administration.
+    '''
+    def __init__(self):
+        gtk.Menu.__init__(self)
+
+        column = 0
+        row_max = 14
+
+        settings_source = SettingsSource()
+        for source in settings_source.get_settings_menu_tree().get_toplevel_source_list():
+            menu_items = []
+
+            for item in source.get_items():
+                img = gtk.Image()
+                img.set_from_pixbuf(item.get_icon(gtk.ICON_SIZE_LARGE_TOOLBAR))
+                menu_item = gtk.ImageMenuItem(item.get_name())
+                menu_item.set_image(img)
+                menu_item.connect("activate", lambda mi, i: i.open(), item)
+                menu_item.show()
+                menu_items.append(menu_item)
+
+            for idx in range(len(menu_items)):
+                mi_column = column + int(idx / row_max)
+                mi_row = idx % row_max
+
+                # Offset 2 rows for header and separator menu item
+                self.attach(menu_items[idx],
+                            mi_column, mi_column + 1,
+                            mi_row + 2, mi_row + 3)
+
+            col_cnt = idx / row_max
+            if idx % row_max > 0:
+                col_cnt = col_cnt + 1
+
+            img = gtk.Image()
+            img.set_from_pixbuf(source.get_icon(gtk.ICON_SIZE_LARGE_TOOLBAR))
+            heading_item = gtk.ImageMenuItem(source.get_name())
+            heading_item.set_image(img)
+            heading_item.set_sensitive(False)
+            heading_item.show()
+            self.attach(heading_item, column, column + col_cnt, 0, 1);
+
+            sep = gtk.SeparatorMenuItem()
+            sep.show()
+            self.attach(sep, column, column + col_cnt, 1, 2);
+
+            column = column + col_cnt
+
+
+class SettingsToolMenuButton(ToolMenuButton):
+    def __init__(self, tooltips):
+        img = icon_factory.load_image(gtk.STOCK_PREFERENCES, gtk.ICON_SIZE_LARGE_TOOLBAR)
+        ToolMenuButton.__init__(self, img, _("Settings"))
+        self.set_tooltip(tooltips, _("Preferences & Administration"))
+        self.set_menu(SettingsMenu())
+
+
+#
+# Topic Implementation
+#
+
 class ComputerTopic(Topic):
     '''
     Lists a heterogeneous set of browsable system items, such as devices/drives,
@@ -438,76 +706,41 @@ class ComputerTopic(Topic):
                                                   platform.version())[0] or _("Computer")
         Topic.__init__(self, system_alias_name, uri="topic://Computer")
 
-        # Get the user's real name from /etc/passwd
-        pwent = pwd.getpwuid(os.getuid())
-        self.username = pwent.pw_gecos.split(",")[0] or pwent.pw_name
-
         from gimmie_running import ComputerRunningSource
         self.set_running_source_factory(lambda: ComputerRunningSource())
 
         self.update_time_timeout_id = None
         self.current_time = 0
 
-        self.settings_menu_tree = None
-        try:
-            # Gnome 2.17.91+ changed the file name arbitrarily.  Yay.
-            self.settings_menu_tree = MenuTree("gnomecc.menu")
-        except ValueError:
-            try:
-                # This is the file panel menu-based gnomecc uses.  It generally
-                # includes preferences.menu and system-settings.menu.
-                self.settings_menu_tree = MenuTree("settings.menu")
-            except ValueError:
-                try:
-                    # Some distros rename settings.menu.
-                    self.settings_menu_tree = MenuTree("gnome-settings.menu")
-                except ValueError:
-                    pass
-
-        self.face_monitor = FileMonitor(os.path.expanduser("~/.face"))
-        self.face_monitor.open()
-
     def do_reload(self):
-        places = PlacesSource()
-        devices = DevicesSource()
-        network = BonjourSource()
-
-        if gnomecups:
-            printers = PrinterSource()
-        else:
-            printers = DisabledItemSource(_("Printers"), gtk.STOCK_PRINT, uri="source://Printers")
-
         if gimmie_is_panel_applet():
             ### Show the Favorites source if in applet mode.  This is because
             ### applet mode doesn't have a dock for favorites.
-            source_list = [FavoritesSource(),
-                           "---"] # separator
+            fav = [FavoritesSource(),
+                   "---"] # separator
         else:
-            source_list = []
+            fav = []
 
-        source_list += [RecentComputerItems((devices, places, printers, network)),
-                        None,
-                        places,
-                        devices,
-                        network,
-                        printers,
-                        None]
+        settings = SettingsSource()
+        source_list = [PlacesSource(),
+                       DevicesSource(),
+                       BonjourSource(),
+                       PrinterSource(),
+                       None,
+                       settings]
 
-        if self.settings_menu_tree:
-            settings_src = self.settings_menu_tree.get_toplevel_flat_source()
-            settings_src.get_name = lambda: _("Settings")
-            settings_src.get_uri = lambda: "source://Settings"
-            settings_src.always_show_descriptions = lambda: True
-            source_list.append(settings_src)
+        if not settings.has_administration():
+            try:
+                source_list.append(AdministrationSource())
+            except ValueError, err:
+                print " !!! Error loading Administration items:", err
+        
+        ### Uncomment to list settings inside their toplevel folders in the sidebar
+        #source_list += SettingsSource().get_toplevel_source_list()
+        #source_list.append(None)
 
-            ### Uncomment to list settings inside their toplevel folders in the sidebar
-            #source_list += self.settings_menu_tree.get_toplevel_source_list()
-            #source_list.append(None)
-
-        if os.environ.has_key("GIMMIE_TIMELINE"):
-            source_list += [None,
-                            DisabledItemSource(_("Timeline"), "stock_timer",
-                                               uri="source://Timeline")]
+        recent = RecentAggregate(sources=[x for x in source_list if x])
+        source_list = fav + [recent, None] + source_list
 
         self.set_sidebar_source_list(source_list)
 
@@ -560,187 +793,38 @@ class ComputerTopic(Topic):
             time_btn.set_text(now_str)
         return True
 
-    def _dot_face_changed(self, monitor, uri, img):
-        face = os.path.expanduser("~/.face")
-        if os.path.exists(face):
-            img.set_from_pixbuf(icon_factory.load_icon(face, gtk.ICON_SIZE_LARGE_TOOLBAR))
-        else:
-            img.set_from_pixbuf(icon_factory.load_icon("stock_person",
-                                                       gtk.ICON_SIZE_LARGE_TOOLBAR))
-
     def get_toolbar_items(self, tooltips):
         tools = []
 
-        img = gtk.Image()
-        self.face_monitor.connect("created", self._dot_face_changed, img)
-        self.face_monitor.connect("deleted", self._dot_face_changed, img)
-        self.face_monitor.connect("changed", self._dot_face_changed, img)
-        self._dot_face_changed(None, None, img)
-        
-        btn = ToolMenuButton(img, self.username)
-        btn.set_tooltip(tooltips, _("Switch to another user"))
+        btn = UserToolMenuButton(tooltips)
         btn.set_is_important(True)
-        menu = gtk.Menu()
-        self._add_user_items(menu)
-        btn.set_menu(menu)
         tools.append(btn)
 
-        '''
-        img = icon_factory.load_image(gtk.STOCK_NETWORK, gtk.ICON_SIZE_LARGE_TOOLBAR)
-        btn = ToolMenuButton(img, "")
-        btn.set_tooltip(tooltips, _("Change the network location"))
-        btn.set_is_important(True)
-        menu = gtk.Menu()
-        mi = gtk.MenuItem(_("Not Implemented"))
-        mi.set_sensitive(False)
-        mi.show()
-        menu.append(mi)
-        btn.set_menu(menu)
-        tools.append(btn)
-        '''
+        ### Uncomment to show settings in a columned menu
+        #btn = SettingsToolMenuButton(tooltips)
+        #btn.set_is_important(True)
+        #tools.append(btn)
 
-        if not gimmie_is_panel_applet():
-            img = icon_factory.load_image("gnome-audio", gtk.ICON_SIZE_LARGE_TOOLBAR)
-            btn = ToolMenuButton(img, "")
-            btn.set_tooltip(tooltips, _("Volume: 75%"))
-            btn.set_is_important(True)
-            menu = gtk.Menu()
-            mi = gtk.MenuItem(_("Not Implemented"))
-            mi.set_sensitive(False)
-            mi.show()
-            menu.append(mi)
-            btn.set_menu(menu)
-            tools.append(btn)
+        ### Uncomment to show network location chooser (unimplemeneted)
+        #btn = NetworkLocationToolMenuButton(tooltips)
+        #btn.set_is_important(True)
+        #tools.append(btn)
+
+        ### Uncomment to show volume selector in toolbar (unimplemented)
+        #btn = VolumeToolMenuButton(tooltips)
+        #btn.set_is_important(True)
+        #tools.append(btn)
 
         tools.append(None)
 
-        img = icon_factory.load_image("gnome-shutdown", gtk.ICON_SIZE_LARGE_TOOLBAR)
-        btn = gtk.ToolButton(img, _("Shutdown..."))
-        btn.set_tooltip(tooltips, _("Shutdown or suspend this computer"))
+        btn = ShutdownToolButton(tooltips)
         btn.set_is_important(True)
-        btn.connect("clicked", self.logout, LogoutDialog.SHUTDOWN)
         tools.append(btn)
 
         tools.append(None)
 
-        img = icon_factory.load_image(gtk.STOCK_HELP, gtk.ICON_SIZE_LARGE_TOOLBAR)
-        btn = ToolMenuButton(img, _("Help"))
+        btn = HelpToolMenuButton(tooltips)
         btn.set_is_important(True)
-        menu = gtk.Menu()
-        about = gtk.ImageMenuItem(_("About Gnome"))
-        about.set_image(icon_factory.load_image(gtk.STOCK_ABOUT, gtk.ICON_SIZE_LARGE_TOOLBAR))
-        about.connect("activate", lambda w: launcher.launch_command("gnome-about"))
-        about.show()
-        menu.append(about)
-        btn.set_menu(menu)
-        tools.append(btn)
+        tools.append(btn)    
 
         return tools
-
-    def _add_user_items(self, menu):
-        btn = gtk.ImageMenuItem(_("About Me"))
-        btn.set_image(icon_factory.load_image("user-info", gtk.ICON_SIZE_LARGE_TOOLBAR))
-        btn.connect("activate", lambda b: launcher.launch_command("gnome-about-me"))
-        btn.show()
-        menu.append(btn)
-
-        btn = gtk.ImageMenuItem(_("Switch User"))
-        btn.set_image(icon_factory.load_image("gnome-lockscreen", gtk.ICON_SIZE_LARGE_TOOLBAR))
-        btn.connect("activate", self.switch_user)
-        btn.show()
-        menu.append(btn)
-
-        btn = gtk.ImageMenuItem(_("Log Out..."))
-        btn.set_image(icon_factory.load_image("gnome-logout", gtk.ICON_SIZE_LARGE_TOOLBAR))
-        btn.connect("activate", self.logout, LogoutDialog.LOGOUT)
-        btn.show()
-        menu.append(btn)
-
-    def _make_settings_menu(self):
-        column = 0
-        row_max = 14
-
-        menu = gtk.Menu()
-        for source in self.settings_menu_tree.get_toplevel_source_list():
-            menu_items = []
-
-            for item in source.get_items():
-                img = gtk.Image()
-                img.set_from_pixbuf(item.get_icon(gtk.ICON_SIZE_LARGE_TOOLBAR))
-                menu_item = gtk.ImageMenuItem(item.get_name())
-                menu_item.set_image(img)
-                menu_item.connect("activate", lambda mi, i: i.open(), item)
-                menu_item.show()
-                menu_items.append(menu_item)
-
-            for idx in range(len(menu_items)):
-                mi_column = column + int(idx / row_max)
-                mi_row = idx % row_max
-
-                # Offset 2 rows for header and separator menu item
-                menu.attach(menu_items[idx],
-                            mi_column, mi_column + 1,
-                            mi_row + 2, mi_row + 3)
-
-            col_cnt = idx / row_max
-            if idx % row_max > 0:
-                col_cnt = col_cnt + 1
-
-            img = gtk.Image()
-            img.set_from_pixbuf(source.get_icon(gtk.ICON_SIZE_LARGE_TOOLBAR))
-            heading_item = gtk.ImageMenuItem(source.get_name())
-            heading_item.set_image(img)
-            heading_item.set_sensitive(False)
-            heading_item.show()
-            menu.attach(heading_item, column, column + col_cnt, 0, 1);
-
-            sep = gtk.SeparatorMenuItem()
-            sep.show()
-            menu.attach(sep, column, column + col_cnt, 1, 2);
-
-            column = column + col_cnt
-        return menu
-
-    def get_context_menu_items(self):
-        items = []
-
-        if self.settings_menu_tree:
-            img = icon_factory.load_image(gtk.STOCK_PREFERENCES, gtk.ICON_SIZE_LARGE_TOOLBAR)
-            btn = gtk.ImageMenuItem(_("Settings"))
-            btn.set_image(img)
-            btn.set_submenu(self._make_settings_menu())
-            btn.show()
-            items.append(btn)
-
-        img = icon_factory.load_image("stock_person", gtk.ICON_SIZE_LARGE_TOOLBAR)
-        btn = gtk.ImageMenuItem(self.username)
-        btn.set_image(img)
-        menu = gtk.Menu()
-        self._add_user_items(menu)
-        btn.set_submenu(menu)
-        btn.show()
-        items.append(btn)
-
-        img = icon_factory.load_image(gtk.STOCK_NETWORK, gtk.ICON_SIZE_LARGE_TOOLBAR)
-        btn = gtk.ImageMenuItem(_("Network"))
-        btn.set_image(img)
-        menu = gtk.Menu()
-        btn.set_submenu(menu)
-        btn.show()
-        items.append(btn)
-
-        img = icon_factory.load_image("gnome-shutdown", gtk.ICON_SIZE_LARGE_TOOLBAR)
-        btn = gtk.ImageMenuItem(_("Shutdown..."))
-        btn.set_image(img)
-        btn.show()
-        btn.connect("activate", self.logout, LogoutDialog.SHUTDOWN)
-        items.append(btn)
-
-        return items
-
-    def logout(self, button, method):
-        dialog = LogoutDialog(method)
-        dialog.show()
-
-    def switch_user(self, button):
-        gdmclient.new_login()
