@@ -4,8 +4,10 @@ import os
 import re
 import urllib
 import shutil
-from xml.dom.minidom import parse
+import sys
 from gettext import gettext as _
+from xml.dom.minidom import parse
+from xml.parsers.expat import ExpatError
 
 import gobject
 import gtk
@@ -16,102 +18,103 @@ import wnck
 from gimmie_globals import gimmie_is_panel_applet
 from gimmie_base import Item, ItemSource, DisabledItemSource, Topic, gimmie_get_topic_for_uri
 from gimmie_file import FileItem
-from gimmie_recent import RecentlyUsed, RecentlyUsedOfMimeType
+from gimmie_recent import RecentlyUsedOfMimeType, RecentAggregate
 from gimmie_util import *
 from gimmie_trash import trash_monitor
 from gimmie_computer import DriveItem
-from gimmie_tomboy import TomboySource
+from gimmie_tomboy import tomboy_source
 
 
-class RecentAggregate(ItemSource):
-    '''
-    This ItemSource subclass aggregates all the items from a list of
-    ItemSources, by including the first Item encountered of a URI and
-    filtering duplicates.
-    '''
-    def __init__(self, sources):
-        ItemSource.__init__(self, name=_("Recently Used"), icon="stock_calendar")
-
-        # Sources provide the real items we will display
-        self.sources = [x for x in sources if x]
-        for source in self.sources:
-            self._listen_to_source(source)
-
-    def _listen_to_source(self, source):
-        source.connect("reload", lambda x: self.emit("reload"))
-
-    def get_items(self):
-        item_uris = {}
-
-        # Find items matching recent uris
-        for source in self.sources:
-            for item in source.get_items():
-                uri = item.get_uri()
-                # Block special items the source might include like "Create
-                # New Note" and avoid duplicate items for the same URI.
-                if not item.is_special() and uri and uri not in item_uris:
-                    item_uris[uri] = item
-
-        items = item_uris.values()
-
-        return items
-
-    def do_reload(self):
-        pass # Do nothing.
-
+#
+# Sidebar ItemSources
+#
 
 class DirectorySource(ItemSource):
-    def __init__(self, path, icon = "stock_folder"):
+    def __init__(self, path, name = None, icon = "stock_folder"):
         self.path = os.path.expanduser(path)
-        dirname = os.path.basename(self.path)
-        ItemSource.__init__(self, dirname, icon=icon)
+
+        if not name:
+            name = os.path.basename(self.path).decode(sys.getfilesystemencoding(), "replace")
+        ItemSource.__init__(self, name=name, icon=icon)
 
         self.monitor = FileMonitor(self.path)
-        self.monitor.connect("changed", lambda x, file: self.emit("reload"))
+        self.monitor.connect("event", lambda m, uri, ev: self.emit("reload"))
         self.monitor.open()
 
     def get_enabled(self):
         return os.path.exists(self.path)
 
-    def add_name(self, paths, dirname, names):
+    def _add_name(self, paths, dirname, names):
         for name in names:
             paths.append(os.path.join(dirname, name))
 
-    def do_reload(self):
+    def get_items_uncached(self):
         paths = []
-        os.path.walk(self.path, self.add_name, paths)
-        self.set_items([FileItem(x) for x in paths])
+        os.path.walk(self.path, self._add_name, paths)
+        for path in paths:
+            yield FileItem(path)
 
 
-class DownloadsSource(DirectorySource):
-    def __init__(self):
-        if os.environ.has_key("DOWNLOAD_PATH"):
-            path = os.environ["DOWNLOAD_PATH"]
-        else:
-            path = "~/Downloads"
-        DirectorySource.__init__(self, path, icon="stock_internet")
+class EpiphanyDownloadsSource(ItemSource):
+    def __init__(self, name = "Epiphany Downloads"):
+        ItemSource.__init__(self, name=name)
 
-        self.download_rdf_monitors = {}
-        self.do_reload()
+        self.ephy_directory_source = None
+        self.ephy_gconf_bridge = GConfBridge("/apps/epiphany/directories/")
+        self.ephy_gconf_bridge.connect("changed::downloads_folder",
+                                       lambda gb: self._ephy_downloads_folder_changed())
+        self._ephy_downloads_folder_changed()
 
-    def _get_download_rdf_paths(self):
-        return glob.glob(os.path.expanduser("~/.mozilla/*/*.default/downloads.rdf"))
+    def _ephy_downloads_folder_changed(self):
+        dir = self.ephy_gconf_bridge.get("downloads_folder", "")
+        if not dir:
+            return
+
+        self.ephy_directory_source = DirectorySource(dir)
+        self.ephy_directory_source.connect("reload", lambda x: self.emit("reload"))
+        self.emit("reload") # Notify listeners
 
     def get_enabled(self):
-        return DirectorySource.get_enabled(self) or self._get_download_rdf_paths() != []
+        return self.ephy_directory_source and self.ephy_directory_source.get_enabled()
 
-    def do_reload(self):
-        DirectorySource.do_reload(self)
-        dir_items = DirectorySource.get_items(self)
-        
-        for download_rdf in self._get_download_rdf_paths():
+    def get_items(self):
+        if self.ephy_directory_source:
+            for item in self.ephy_directory_source.get_items():
+                yield item
+
+
+class MozillaDownloadsSource(ItemSource):
+    def __init__(self, name = "Mozilla Downloads", icon = None):
+        ItemSource.__init__(self, name=name, icon=icon)
+        self.download_rdf_monitors = {}
+
+    def get_download_rdf_paths(self):
+        '''
+        Returns a list of paths to Mozilla downloads.rdf files.  It only returns
+        files in the "default" profile, to avoid porn.  Maybe this should be
+        revisited.
+        '''
+        return glob.glob(os.path.expanduser("~/.mozilla/*/*default*/downloads.rdf"))
+
+    def get_enabled(self):
+        if not self.download_rdf_monitors:
+            self._monitor_download_rdf_paths()
+        return len(self.download_rdf_monitors) > 0
+
+    def _monitor_download_rdf_paths(self):
+        for download_rdf in self.get_download_rdf_paths():
             if not self.download_rdf_monitors.has_key(download_rdf):
                 # Emit reload on download.rdf change
                 mon = FileMonitor(download_rdf)
-                mon.connect("changed", lambda x, file: self.emit("reload"))
+                mon.connect("event", lambda m, uri, ev: self.emit("reload"))
                 mon.open()
                 self.download_rdf_monitors[download_rdf] = mon
 
+    def get_items_uncached(self):
+        if not self.download_rdf_monitors:
+            self._monitor_download_rdf_paths()
+
+        for download_rdf in self.download_rdf_monitors:
             try:
                 # Read previous download locations
                 download_doc = parse(download_rdf)
@@ -122,31 +125,119 @@ class DownloadsSource(DirectorySource):
                     # Sometimes contains absolute paths instead of URLs
                     if os.path.isabs(uri):
                         uri = "file://" + uri
-                    dir_items.append(FileItem(uri))
-            except IOError, (errno, strerror):
-                print " !!! Error parsing Mozilla %s file: %s" % (download_rdf, strerror)
 
-        self.set_items(dir_items)
+                    yield FileItem(uri)
+            except (IOError, ExpatError), err:
+                print " !!! Error parsing Mozilla download file '%s': %s" % (download_rdf, err)
 
 
-class AddSmartFolder(gtk.Dialog):
+class ThunderbirdAttachmentsSource(MozillaDownloadsSource):
     '''
-    Dialog to choose a stock Smart Folder to add to the sidebar,
-    possibly prompting for some info (e.g. Tag names,
-    modification date, etc).
+    A MozillaDownloadsSource subclass that reads Thunderbird downloads.rdf files.
     '''
-    def __init__(self, parent):
-        gtk.Dialog.__init__(self,
-                            _("New Smart Folder"),
-                            parent,
-                            gtk.DIALOG_DESTROY_WITH_PARENT | gtk.DIALOG_NO_SEPARATOR,
-                            (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
-                             gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
+    def __init__(self):
+        MozillaDownloadsSource.__init__(self, name=_("Attachments"), icon="stock_attach")
 
-    def get_sidebar_source(self):
-        return None
+    def get_download_rdf_paths(self):
+        '''Returns a list of paths to Mozilla Thunderbird downloads.rdf files.'''
+        return glob.glob(os.path.expanduser("~/.*thunderbird/*/downloads.rdf"))
 
 
+class DownloadsSource(RecentAggregate):
+    '''
+    Aggregate ItemSource subclass that lists files from: ~/Downloads,
+    Mozilla/Firefox downloads.rdf files, and the Epiphany download directory.
+    '''
+    def __init__(self, name = _("Downloads"), icon = "stock_internet"):
+        moz_source = MozillaDownloadsSource()
+        ephy_source = EpiphanyDownloadsSource()
+
+        if os.environ.has_key("DOWNLOAD_PATH"):
+            path = os.environ["DOWNLOAD_PATH"]
+        else:
+            path = "~/Downloads"
+        download_dir_source = DirectorySource(path)
+
+        RecentAggregate.__init__(self,
+                                 name=name,
+                                 icon=icon,
+                                 sources=[moz_source, ephy_source, download_dir_source])
+
+
+class RecentlyUsedDocumentsSource(RecentlyUsedOfMimeType):
+    ### FIXME: This is lame, we should generate this list somehow.
+    DOCUMENT_MIMETYPES = [
+        # Covers:
+        #   vnd.corel-draw
+        #   vnd.ms-powerpoint
+        #   vnd.ms-excel
+        #   vnd.oasis.opendocument.*
+        #   vnd.stardivision.*
+        #   vnd.sun.xml.*
+        re.compile("application/vnd.*"),
+        # Covers: x-applix-word, x-applix-spreadsheet, x-applix-presents
+        re.compile("application/x-applix-*"),
+        # Covers: x-kword, x-kspread, x-kpresenter, x-killustrator
+        re.compile("application/x-k(word|spread|presenter|illustrator)"),
+        re.compile("text/*"),
+        re.compile("image/*"),
+        "application/ms-powerpoint",
+        "application/msword",
+        "application/pdf",
+        "application/postscript",
+        "application/ps",
+        "application/rtf",
+        "application/x-abiword",
+        "application/x-asp",
+        "application/x-bittorrent",
+        "application/x-blender",
+        "application/x-cgi",
+        "application/x-dia-diagram",
+        "application/x-dvi",
+        "application/x-glade",
+        "application/x-gnucash",
+        "application/x-gnumeric",
+        "application/x-iso-image",
+        "application/x-jbuilder-project",
+        "application/x-magicpoint",
+        "application/x-mrproject",
+        "application/x-php",
+        ]
+    
+    def __init__(self):
+        RecentlyUsedOfMimeType.__init__(self,
+                                        name=_("Documents"),
+                                        icon="stock_new-presentation",
+                                        mimetype_list=self.DOCUMENT_MIMETYPES)
+
+    def get_items_uncached(self):
+        for item in RecentlyUsedOfMimeType.get_items_uncached(self):
+            yield FileItem(uri=item.get_uri(), timestamp=item.get_timestamp())
+
+
+class RecentlyUsedMediaSource(RecentlyUsedOfMimeType):
+    ### FIXME: This is lame, we should generate this list somehow.
+    MEDIA_MIMETYPES = [
+        re.compile("video/*"),
+        re.compile("audio/*"),
+        "application/ogg"
+        ]
+
+    def __init__(self):
+        RecentlyUsedOfMimeType.__init__(self,
+                                        name=_("Music & Movies"),
+                                        icon="gnome-mime-video",
+                                        mimetype_list=self.MEDIA_MIMETYPES)
+
+    def get_items_uncached(self):
+        for item in RecentlyUsedOfMimeType.get_items_uncached(self):
+            yield FileItem(uri=item.get_uri(), timestamp=item.get_timestamp())
+
+
+#
+# Toolbar Items
+#
+
 class NewFromTemplateDialog(gtk.FileChooserDialog):
     '''
     Dialog to create a new document from a template
@@ -266,6 +357,82 @@ class NewDocumentMenuButton(ToolMenuButton):
         dlg.show()
 
 
+class PlacesMenu(gtk.Menu):
+    def __init__(self):
+        gtk.Menu.__init__(self)
+
+        # Include all the mounted drives
+        computer = gimmie_get_topic_for_uri("topic://Computer")
+        device_source = computer.get_source_for_uri("source://Devices")
+        for item in device_source.get_items():
+            if isinstance(item, DriveItem) and item.get_is_mounted():
+                self.append(self._add_place_item(item.get_name(),
+                                                 item.get_uri(),
+                                                 item.get_icon(gtk.ICON_SIZE_LARGE_TOOLBAR)))
+
+        sep = gtk.SeparatorMenuItem()
+        sep.show()
+        self.append(sep)
+
+        # Include all the Gtk bookmarks
+        for uri, name, mime, icon in places.get_places():
+            self.append(self._add_place_item(name, uri, icon))
+
+        sep = gtk.SeparatorMenuItem()
+        sep.show()
+        self.append(sep)
+
+        # Add the trash
+        if trash_monitor.is_empty():
+            self.append(self._add_place_item(_("Trash"), "trash://", "gnome-fs-trash-empty"))
+        else:
+            self.append(self._add_place_item(_("Trash"), "trash://", "gnome-fs-trash-full"))
+
+    def _add_place_item(self, name, uri, icon_name = "gnome-fs-directory"):
+        menu_item = gtk.ImageMenuItem(name)
+        menu_item.set_image(icon_factory.load_image(icon_name, gtk.ICON_SIZE_LARGE_TOOLBAR))
+        menu_item.show()
+        menu_item.connect("activate", lambda mi, uri: launcher.launch_uri(uri), uri)
+        return menu_item
+
+
+class PlacesMenuButton(ToolMenuButton):
+    def __init__(self, tooltips):
+        img = icon_factory.load_image("gnome-fs-bookmark", gtk.ICON_SIZE_LARGE_TOOLBAR)
+        ToolMenuButton.__init__(self, img, _("Places"))
+        self.set_tooltip(tooltips, _("Open commonly used locations"))
+        self.set_menu(PlacesMenu())
+
+
+class TrashButton(gtk.ToolButton):
+    def __init__(self, tooltips):
+        img = icon_factory.load_image("gnome-fs-trash-full", gtk.ICON_SIZE_LARGE_TOOLBAR)
+        gtk.ToolButton.__init__(self, img, _("Trash"))
+        self.set_tooltip(tooltips, _("Open the deleted files trashcan"))
+        self.connect("clicked", lambda x: launcher.launch_uri("trash://"))
+
+
+class PrintersMenuButton(gtk.ToolButton):
+    __gsignals__ = {
+        "clicked" : "override"
+        }
+
+    def __init__(self, tooltips):
+        img = icon_factory.load_image(gtk.STOCK_PRINT, gtk.ICON_SIZE_LARGE_TOOLBAR)
+        gtk.ToolButton.__init__(self, img, _("Printers"))
+        self.set_tooltip(tooltips, _("View printers available for document printing"))
+
+    def do_clicked(self):
+        topic = gimmie_get_topic_for_uri("topic://Computer")
+        topicwin = topic.get_topic_window()
+        topicwin.set_source_by_uri("source://Printers")
+        topicwin.present()
+
+
+#
+# Topic Implementation
+#
+
 class DocumentsTopic(Topic):
     '''
     Lists recently opened/edited documents from ~/.recently-used, and some other
@@ -279,117 +446,25 @@ class DocumentsTopic(Topic):
         from gimmie_running import RunningDocuments
         self.set_running_source_factory(lambda: RunningDocuments())
 
-        # Tomboy loads notes in a worker thread, so begin it early.
-        self.tomboy_source = TomboySource()
-
     def do_reload(self):
         source_list = [None,
-                       RecentlyUsedOfMimeType(
-                           _("Documents"),
-                           "stock_new-presentation",
-                           [  # Covers:
-                              #   vnd.corel-draw
-                              #   vnd.ms-powerpoint
-                              #   vnd.ms-excel
-                              #   vnd.oasis.opendocument.*
-                              #   vnd.stardivision.*
-                              #   vnd.sun.xml.*
-                            re.compile("application/vnd.*"),
-                              # Covers: x-applix-word, x-applix-spreadsheet, x-applix-presents
-                            re.compile("application/x-applix-*"),
-                              # Covers: x-kword, x-kspread, x-kpresenter, x-killustrator
-                            re.compile("application/x-k(word|spread|presenter|illustrator)"),
-                            re.compile("text/*"),
-                            re.compile("image/*"),
-                            "application/ms-powerpoint",
-                            "application/msword",
-                            "application/pdf",
-                            "application/postscript",
-                            "application/ps",
-                            "application/rtf",
-                            "application/x-abiword",
-                            "application/x-asp",
-                            "application/x-bittorrent",
-                            "application/x-blender",
-                            "application/x-cgi",
-                            "application/x-dia-diagram",
-                            "application/x-dvi",
-                            "application/x-glade",
-                            "application/x-gnucash",
-                            "application/x-gnumeric",
-                            "application/x-iso-image",
-                            "application/x-jbuilder-project",
-                            "application/x-magicpoint",
-                            "application/x-mrproject",
-                            "application/x-php",
-                           ]),
-                       self.tomboy_source, # Tomboy
-                       RecentlyUsedOfMimeType(_("Music & Movies"),
-                                              "gnome-mime-video",
-                                              [re.compile("video/*"),
-                                               re.compile("audio/*"),
-                                               "application/ogg"]),
-                       DisabledItemSource(_("F-Spot Photos"), "f-spot-logo"),
+                       RecentlyUsedDocumentsSource(),
+                       tomboy_source, # Tomboy
+                       RecentlyUsedMediaSource(),
+                       DisabledItemSource(_("Flickr Photos"), "emblem-pictures"),
+                       DisabledItemSource(_("Google Office"), "stock_new-spreadsheet"),
                        None,
                        DownloadsSource(),
-                       DisabledItemSource(_("Attachments"), "stock_attach"),
+                       ThunderbirdAttachmentsSource()
                        ]
         
-        source_list.insert(0, RecentAggregate(source_list))
-        
+        source_list.insert(0, RecentAggregate(sources=[x for x in source_list if x]))
+
         self.set_sidebar_source_list(source_list)
 
     def get_hint_color(self):
         return gtk.gdk.color_parse("lightgreen")
 
-    def _add_place_item(self,
-                        menu,
-                        name,
-                        uri,
-                        icon_name = "gnome-fs-directory"):
-        menu_item = gtk.ImageMenuItem(name)
-        menu_item.set_image(icon_factory.load_image(icon_name, gtk.ICON_SIZE_LARGE_TOOLBAR))
-        menu_item.show()
-        menu_item.connect("activate", lambda mi, uri: launcher.launch_uri(uri), uri)
-        return menu_item
-
-    def _add_places(self, menu):
-        # Include all the mounted drives
-        computer = gimmie_get_topic_for_uri("topic://Computer")
-        device_source = computer.get_source_for_uri("source://Devices")
-        for item in device_source.get_items():
-            if isinstance(item, DriveItem) and item.get_is_mounted():
-                menu.append(self._add_place_item(menu,
-                                                 item.get_name(),
-                                                 item.get_uri(),
-                                                 item.get_icon(gtk.ICON_SIZE_LARGE_TOOLBAR)))
-
-        sep = gtk.SeparatorMenuItem()
-        sep.show()
-        menu.append(sep)
-
-        # Include all the Gtk bookmarks
-        for uri, name, mime, icon in places.get_places():
-            menu.append(self._add_place_item(menu, name, uri, icon))
-
-        sep = gtk.SeparatorMenuItem()
-        sep.show()
-        menu.append(sep)
-
-        # Add the trash
-        if trash_monitor.is_empty():
-            menu.append(self._add_place_item(menu, _("Trash"), "trash://", "gnome-fs-trash-empty"))
-        else:
-            menu.append(self._add_place_item(menu, _("Trash"), "trash://", "gnome-fs-trash-full"))
-
-    def _show_add_smart_folder_dialog(self, b):
-        dlg = AddSmartFolder(b.get_toplevel())
-        if dlg.run() == gtk.RESPONSE_ACCEPT:
-            source = dlg.get_sidebar_source()
-            ### FIXME: Add to sources sidebar
-            print "Adding!"
-        dlg.destroy()
-
     def get_toolbar_items(self, tooltips):
         tools = []
 
@@ -397,80 +472,19 @@ class DocumentsTopic(Topic):
         btn.set_is_important(True)
         tools.append(btn)
 
-        '''
-        img = icon_factory.load_image("stock_new-dir", gtk.ICON_SIZE_LARGE_TOOLBAR)
-        btn = gtk.ToolButton(img, _("New Smart Folder"))
-        btn.set_tooltip(tooltips, _("Add a new smart folder to the sidebar"))
+        btn = PlacesMenuButton(tooltips)
         btn.set_is_important(True)
-        btn.connect("clicked", self._show_add_smart_folder_dialog)
-        tools.append(btn)
-        '''
-
-        img = icon_factory.load_image("gnome-fs-bookmark", gtk.ICON_SIZE_LARGE_TOOLBAR)
-        btn = ToolMenuButton(img, _("Folders"))
-        btn.set_tooltip(tooltips, _("Open commonly used document locations"))
-        btn.set_is_important(True)
-        menu = gtk.Menu()
-        self._add_places(menu)
-        btn.set_menu(menu)
         tools.append(btn)
 
-        ### FIXME: Include this?
-        '''
-        img = icon_factory.load_image("gnome-fs-trash-full", gtk.ICON_SIZE_LARGE_TOOLBAR)
-        btn = gtk.ToolButton(img, _("Trash"))
-        btn.set_tooltip(tooltips, _("Open the deleted files trashcan"))
-        btn.set_is_important(True)
-        btn.connect("clicked", lambda x: launcher.launch_uri("trash://"))
-        tools.append(btn)
-        '''
+        ### Uncomment to include a trashcan in the toolbar
+        #btn = TrashButton(tooltips)
+        #btn.set_is_important(True)
+        #tools.append(btn)
 
         if not gimmie_is_panel_applet():
-            tools.append(None)
-
-            img = icon_factory.load_image(gtk.STOCK_PRINT, gtk.ICON_SIZE_LARGE_TOOLBAR)
-            btn = gtk.ToolButton(img, _("Printers"))
-            btn.set_tooltip(tooltips, _("View printers available for document printing"))
+            tools.append(None) # Spacer
+            btn = PrintersMenuButton(tooltips)
             btn.set_is_important(True)
-            btn.connect("clicked", lambda x: self._open_printers())
             tools.append(btn)
 
         return tools
-
-    def _open_printers(self):
-        topic = gimmie_get_topic_for_uri("topic://Computer")
-        topicwin = topic.get_topic_window()
-        topicwin.set_source_by_uri("source://Printers")
-        topicwin.present()
-
-    def get_context_menu_items(self):
-        items = []
-
-        img = icon_factory.load_image("stock_new-template", gtk.ICON_SIZE_LARGE_TOOLBAR)
-        btn = gtk.ImageMenuItem(_("New Document"))
-        btn.set_image(img)
-        menu = gtk.Menu()
-        self._add_templates(menu)
-        btn.set_submenu(menu)
-        btn.show_all()
-        items.append(btn)
-
-        img = icon_factory.load_image("gnome-fs-bookmark", gtk.ICON_SIZE_LARGE_TOOLBAR)
-        btn = gtk.ImageMenuItem(_("Go"))
-        btn.set_image(img)
-        menu = gtk.Menu()
-        self._add_places(menu)
-        btn.set_submenu(menu)
-        btn.show_all()
-        items.append(btn)
-
-        items.append(None)
-
-        img = icon_factory.load_image(gtk.STOCK_PRINT, gtk.ICON_SIZE_LARGE_TOOLBAR)
-        btn = gtk.ImageMenuItem(_("Printers"))
-        btn.set_image(img)
-        btn.connect("activate", lambda x: self._open_printers())
-        btn.show_all()
-        items.append(btn)
-        
-        return items
