@@ -312,7 +312,7 @@ class TopicRunningList(gtk.EventBox):
         print " *** Reloading TopicRunningList:", source.get_name()
 
         target_height = self.get_best_icon_size_for_screen()
-        new_items = source.get_items()
+        new_items = list(source.get_items())
         
         for i in self.content:
             if i in new_items:
@@ -745,6 +745,7 @@ class FriendlyPager(gtk.VBox):
         gtk.VBox.__init__(self, False, 0)
         self.use_compact_pager = False
         self.orientation = None
+        self.viewport_geom = None
 
         self.labels = gtk.HBox(True, 0)
         self.labels.show()
@@ -809,7 +810,17 @@ class FriendlyPager(gtk.VBox):
 
             self._toggle_compact(screen.get_workspace_count() > 4)
             
+            self.viewport_geom = None
+            hide_pager = False
             if (cnt == 1):
+                # Only hide the pager if viewports aren't used.
+                window = gtk.gdk.get_default_root_window()
+                type, format, self.viewport_geom = window.property_get("_NET_DESKTOP_GEOMETRY", "CARDINAL")
+                if (self.viewport_geom[0] <= screen.get_width()) and \
+                   (self.viewport_geom[1] <= screen.get_height()):
+                    hide_pager = True
+            
+            if hide_pager:
                 self.hide()
             else:
                 self.show()
@@ -820,22 +831,37 @@ class FriendlyPager(gtk.VBox):
         workspace = screen.get_active_workspace()
         if not workspace:
             return
-
-        idx = workspace.get_number()
-        max_idx = screen.get_workspace_count() - 1
-        if ev.direction == gtk.gdk.SCROLL_DOWN:
-            d = 1
+        
+        if screen.get_workspace_count() == 1:
+            screen_width = screen.get_width()
+            if self.viewport_geom and (self.viewport_geom[0] > screen_width):
+                workspace = screen.get_active_workspace()
+    
+                if ev.direction == gtk.gdk.SCROLL_DOWN:
+                    x = workspace.get_viewport_x() + screen_width;
+                else:
+                    x = workspace.get_viewport_x() - screen_width
+                    if x < 0:
+                        x = self.viewport_geom[0] - screen_width
+                
+                screen.move_viewport(x, screen.get_height())
         else:
-            d = -1
-
-        workspace = screen.get_workspace(idx + d)
-        if not workspace:
-            if d == 1:
-                workspace = screen.get_workspace(0)
+            
+            idx = workspace.get_number()
+            max_idx = screen.get_workspace_count() - 1
+            if ev.direction == gtk.gdk.SCROLL_DOWN:
+                d = 1
             else:
-                workspace = screen.get_workspace(max_idx)
-
-        workspace.activate(ev.time)
+                d = -1
+    
+            workspace = screen.get_workspace(idx + d)
+            if not workspace:
+                if d == 1:
+                    workspace = screen.get_workspace(0)
+                else:
+                    workspace = screen.get_workspace(max_idx)
+    
+            workspace.activate(ev.time)
 
     def _pager_button_press(self, pager, ev):
         if ev.button == 3:
