@@ -151,7 +151,7 @@ class RunningItem(Item):
 
 class RunningItemSource(ItemSource):
     def __init__(self, name):
-        ItemSource.__init__(self, name, None)
+        ItemSource.__init__(self, name=name)
 
         self.screen = wnck.screen_get(0)
         self.timeout_id = None
@@ -194,7 +194,7 @@ from gimmie_util import launcher
 
 class RunningApplications(RunningItemSource):
     def __init__(self):
-        RunningItemSource.__init__(self, "Running Applications")
+        RunningItemSource.__init__(self, name="Running Applications")
 
     def munge_command_and_wmclass(self, cmd, app):
         split = cmd.split()
@@ -362,7 +362,7 @@ from gimmie_util import gconf_bridge
 
 class ComputerRunningSource(ItemSource, IOrientationAware):
     def __init__(self):
-        ItemSource.__init__(self, "Computer Running Source", None)
+        ItemSource.__init__(self, name="Computer Running Source")
         self.pager = None
         self.orientation = None
 
@@ -435,7 +435,7 @@ class RunningDocuments(RunningItemSource):
     documents opened via the Documents window.
     '''
     def __init__(self):
-        RunningItemSource.__init__(self, "Opened Documents")
+        RunningItemSource.__init__(self, name="Opened Documents")
 
     def get_bookmark_items(self):
         items = []
@@ -563,7 +563,7 @@ class RunningChats(RunningItemSource):
     control plugin.
     '''
     def __init__(self):
-        RunningItemSource.__init__(self, "Active Conversations")
+        RunningItemSource.__init__(self, name="Active Conversations")
 
         # Listen for changes in the Gaim files
         gaim_reader.connect("reload", lambda x: self.emit("reload"))
