@@ -1,5 +1,6 @@
 
 import datetime
+import math
 from xml.sax import saxutils
 from gettext import gettext as _
 
@@ -14,7 +15,7 @@ except ImportError:
     pass
 
 from gimmie_base import ItemSource, gimmie_get_topic_for_uri
-from gimmie_util import icon_factory, ToolMenuButton, gconf_bridge
+from gimmie_util import bookmarks, icon_factory, ToolMenuButton, gconf_bridge
 from gimmie_globals import gimmie_is_panel_applet
 
 
@@ -71,7 +72,7 @@ class SearchToolItem(gtk.ToolItem):
             self.entry = gtk.Entry()
             box.pack_start(self.entry, False, False, 10)
 
-        self.entry.set_width_chars(18)
+        self.entry.set_width_chars(14)
         self.entry.set_text(self.default_search_text)
         self.entry.show()
         self.entry.connect("activate", lambda w: self._typing_timeout())
@@ -158,10 +159,11 @@ class ZoomMenuToolItem(ToolMenuButton):
 
     _zoomlist = [[_("Today"), 1],
                  [_("This week"), 7],
+                 [_("Last 2 weeks"), 14],
                  [_("This month"), 30],
                  [_("Last 2 months"), 60],
                  [_("Last 3 months"), 90],
-                 [_("Forever"), -1]]
+                 [_("Last 6 months"), 180]]
     _default_zoom = 7
 
     def __init__(self):
@@ -207,9 +209,8 @@ class ZoomMenuToolItem(ToolMenuButton):
     def get_zoom_level(self):
         return self.zoom_level
 
-    def get_zoom_level_list(self, floor = _default_zoom, include_forever = True):
-        return [days for name, days in self._zoomlist \
-                if days >= floor or (days < 0 and include_forever)]
+    def get_zoom_level_list(self, floor = _default_zoom):
+        return [days for name, days in self._zoomlist if days >= floor]
 
     def set_zoom_level(self, zoom_level):
         self.zoom_level = zoom_level
@@ -237,25 +238,34 @@ class SparkLine(gtk.EventBox):
 
         self.zoom_level_list = zoom_level_list
         self.zoom_level_list.sort()
+        self.first_day = datetime.timedelta(self.zoom_level_list[-1])
 
         self.num_days_selected = -1
         self.set_dates([])
 
+    def do_size_request(self, req):
+        req.width = self.first_day.days
+        req.height = 24
+
     def do_realize(self):
         gtk.EventBox.do_realize(self)
         self.window.set_background(self.get_style().base[self.state])
         self.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.HAND1))
 
     def do_button_press_event(self, ev):
-        first_date_ordinal = self.first_date.toordinal()
         today_ordinal = datetime.date.today().toordinal()
+        first_ordinal = (datetime.date.today() - self.first_day).toordinal()
 
-        num_days = today_ordinal - first_date_ordinal
+        num_days = today_ordinal - first_ordinal
         if num_days == 0:
             # No data, nothing to click
             return True
 
         day_width = self.allocation.width / num_days
+        if day_width == 0:
+            # Too much data, nothing to click
+            return True
+
         click_day = num_days - (ev.x / day_width)
 
         for i in self.zoom_level_list:
@@ -267,16 +277,14 @@ class SparkLine(gtk.EventBox):
         return True
 
     def do_expose_event(self, ev):
-        first_date_ordinal = self.first_date.toordinal()
         today_ordinal = datetime.date.today().toordinal()
+        first_ordinal = (datetime.date.today() - self.first_day).toordinal()
 
-        num_days = today_ordinal - first_date_ordinal
+        num_days = today_ordinal - first_ordinal
         if num_days == 0:
             # No data, nothing to draw
             return True
 
-        print " *** SparkLine: Redrawing with %s days of history" % num_days
-
         # Width of individual day bars, and total of all bars
         day_width = self.allocation.width / num_days
         total_width = day_width * num_days
@@ -309,7 +317,7 @@ class SparkLine(gtk.EventBox):
         levels_list.append(num_days)
         for i in levels_list:
             if i <= num_days:
-                x_offset = (today_ordinal - first_date_ordinal - i) * day_width
+                x_offset = (today_ordinal - first_ordinal - i) * day_width
                 self.window.draw_rectangle(baseline_gc, True,
                                            x_offset, base_height,
                                            1, 5)
@@ -319,14 +327,14 @@ class SparkLine(gtk.EventBox):
                                    total_width, base_height,
                                    1, 5)
 
-        one_item_height = 2
-        if self.largest_cnt > 0:
-            one_item_height = base_height / self.largest_cnt
-
         # Draw the data bars
         for date, cnt in self.date_cnts.iteritems():
-            x_offset = (date.toordinal() - first_date_ordinal - 1) * day_width
-            height = one_item_height * cnt
+            if date.toordinal() < first_ordinal:
+                continue
+
+            x_offset = (date.toordinal() - first_ordinal - 1) * day_width
+            height = int(math.log(base_height) * cnt)
+            height = min(height, base_height)
 
             gc = bar_gc
             if today_ordinal - date.toordinal() < self.num_days_selected:
@@ -344,8 +352,7 @@ class SparkLine(gtk.EventBox):
     def set_dates(self, datetime_list):
         self.date_cnts = {}
         self.largest_cnt = 0
-        self.first_date = datetime.date.today()
-        
+
         for date in datetime_list:
             if isinstance(date, datetime.datetime):
                 date = date.date()
@@ -354,13 +361,10 @@ class SparkLine(gtk.EventBox):
 
             new_cnt = self.date_cnts.get(date, 0) + 1
             self.date_cnts[date] = new_cnt
-        
+
             if new_cnt > self.largest_cnt:
                 self.largest_cnt = new_cnt
 
-            if date < self.first_date:
-                self.first_date = date
-
         self.queue_draw()
 
     def set_zoom_level(self, zoom_level):
@@ -377,10 +381,11 @@ class TimeBar(gtk.EventBox):
 
     _zoomlist = [[_("Today"), 1],
                  [_("This week"), 7],
+                 [_("Last 2 weeks"), 14],
                  [_("This month"), 30],
                  [_("Last 2 months"), 60],
                  [_("Last 3 months"), 90],
-                 [_("Forever"), -1]]
+                 [_("Last 6 months"), 180]]
     _default_zoom = 7
 
     def __init__(self):
@@ -391,16 +396,17 @@ class TimeBar(gtk.EventBox):
         box.show()
         self.add(box)
 
-        self.spark = SparkLine(self.get_zoom_level_list(floor=0, include_forever=False))
+        self.spark = SparkLine(self.get_zoom_level_list(floor=0))
         self.spark.connect("zoom-changed", lambda w, num_days: self.set_zoom_level(num_days))
 
         align = gtk.Alignment(0.5, 0.5, 1.0, 1.0)
         align.set_property("top-padding", 4)
-        align.set_property("bottom-padding", 4)
         align.add(self.spark)
         box.pack_start(align, True, True, 6)
 
-        self.label = gtk.Label("Last 2 weeks")
+        self.label = gtk.Label()
+        self.label.set_width_chars(max([len(x[0]) for x in self._zoomlist]))
+        self.label.set_alignment(1.0, 0.5)
         box.pack_start(self.label, False, False, 6)
 
         self.in_btn = gtk.Button()
@@ -417,14 +423,13 @@ class TimeBar(gtk.EventBox):
 
         box.show_all()
 
-        self.zoom_level = self._default_zoom
+        self.set_zoom_level(self._default_zoom)
 
     def get_zoom_level(self):
         return self.zoom_level
 
-    def get_zoom_level_list(self, floor = _default_zoom, include_forever = True):
-        return [days for name, days in self._zoomlist \
-                if days >= floor or (days < 0 and include_forever)]
+    def get_zoom_level_list(self, floor = _default_zoom):
+        return [days for name, days in self._zoomlist if days >= floor]
 
     def _zoom_clicked(self, dir):
         for i in range(len(self._zoomlist)):
@@ -432,10 +437,10 @@ class TimeBar(gtk.EventBox):
             if days == self.zoom_level:
                 try:
                     label, days = self._zoomlist[i + dir]
-                    self.set_zoom_level(days, label)
-                    break
                 except IndexError:
-                    pass
+                    label, days = self._zoomlist[0]
+                self.set_zoom_level(days, label)
+                break
 
     def set_zoom_level(self, zoom_level, label=None):
         self.zoom_level = zoom_level
@@ -529,6 +534,7 @@ class ItemIconView(gtk.IconView):
         self.connect("button-press-event", self._show_item_popup)
         self.connect("button-release-event", self._button_release)
         self.connect("drag-data-get", self._item_drag_data_get)
+        self.connect("motion-notify-event", self._motion_notify)
         self.enable_model_drag_source(0,
                                       [("text/uri-list", 0, 100)],
                                       gtk.gdk.ACTION_LINK | gtk.gdk.ACTION_COPY)
@@ -540,6 +546,12 @@ class ItemIconView(gtk.IconView):
         if self.click_policy == "nautilus":
             self._connect_nautilus_click_policy()
 
+        bookmarks.connect("reload", lambda b: self._reload_visible_items())
+
+        self.highlight_path = None
+        self.highlight_normal_icon = None
+        self.highlight_timeout = 0
+
     def _connect_nautilus_click_policy(self):
         nautilus_dir = "/apps/nautilus/preferences/"
         self.nautilus_click_policy_key = nautilus_dir + "click_policy"
@@ -576,7 +588,7 @@ class ItemIconView(gtk.IconView):
         return not item1 and 1 or \
                not item2 and -1 or \
                cmp(not item1.is_special(), not item2.is_special()) or \
-               cmp(item1.get_name().lower(), item2.get_name().lower())
+               cmp((item1.get_name() or "").lower(), (item2.get_name() or "").lower())
 
     def load_items(self, items, ondone_cb = None):
         '''
@@ -605,6 +617,12 @@ class ItemIconView(gtk.IconView):
         self.unselect_all()
         self.set_model(filterstore) # Set the model
 
+        # Clear highlight state
+        self.highlight_path = None
+        self.highlight_normal_icon = None
+        if self.highlight_timeout != 0:
+            gobject.source_remove(self.highlight_timeout)
+        
         if self.idle_load_id:
             gobject.source_remove(self.idle_load_id) # Cancel pending load
         self.idle_load_id = \
@@ -616,11 +634,11 @@ class ItemIconView(gtk.IconView):
         setting the IconView model and yielding False when finished.
         '''
         for i in items:
-            iter = store.append((i.get_name_markup(), None, i, i.get_is_user_visible()))
+            iter = store.append((None, None, i, i.get_is_user_visible()))
             self.reload_handlers[i] = i.connect("reload", self._reload_item, iter, store)
             yield True # Keep going
 
-        self._reload_all_items(store)
+        self._reload_visible_items()
 
         if ondone_cb:
             ondone_cb()
@@ -638,8 +656,14 @@ class ItemIconView(gtk.IconView):
             icon_size = self._set_item(item, iter, store)
             self._update_sizing(icon_size)
 
-    def _reload_all_items(self, store):
+    def _reload_visible_items(self):
         max_size = 0
+
+        store = self.get_model()
+        if isinstance(store, gtk.TreeModelFilter):
+            # Get a writable model if a filter model is set
+            store = store.get_model()
+
         for row in store:
             if row[3]: # Only update visible items
                 icon_size = self._set_item(row[2], row.iter, store)
@@ -746,6 +770,59 @@ class ItemIconView(gtk.IconView):
             if path:
                 self.item_activated(path)
 
+    def _undo_highlight(self):
+        model = self.get_model()
+        iter = model.convert_iter_to_child_iter(model.get_iter(self.highlight_path))
+
+        # Reset the normal icon
+        model.get_model().set_value(iter, 1, self.highlight_normal_icon)
+
+        self.highlight_path = None
+        self.highlight_normal_icon = None
+
+    def _do_highlight(self, path):
+        model = self.get_model()
+        iter = model.convert_iter_to_child_iter(model.get_iter(path))
+
+        pixbuf = model.get_model().get_value(iter, 1)
+        if pixbuf:
+            # Set highlight icon
+            model.get_model().set_value(iter, 1, icon_factory.colorshift(pixbuf, 30))
+
+            # Store the path and regular icon for later
+            self.highlight_path = path
+            self.highlight_normal_icon = pixbuf
+
+    def _motion_notify_timeout(self):
+        x, y = self.get_pointer()
+        path = self.get_path_at_pos(x, y)
+
+        if path != self.highlight_path:
+            if self.highlight_path:
+                self._undo_highlight()
+            if path:
+                self._do_highlight(path)
+
+        if self.click_policy == "single":
+            cursor = None
+            if path:
+                # Set hand cursor
+                cursor = gtk.gdk.Cursor(gtk.gdk.HAND2)
+            self.window.set_cursor(cursor)
+
+        self.highlight_timeout = 0
+        return False
+
+    def _motion_notify(self, view, ev):
+        if self.highlight_timeout != 0:
+            gobject.source_remove(self.highlight_timeout)
+
+        # Avoid highlighting while rubberbanding
+        if not ev.state & gtk.gdk.BUTTON1_MASK:
+            ### FIXME: IconView flickers if we replace the icon too quickly, so use
+            ###        a short timeout.
+            self.highlight_timeout = gobject.timeout_add(5, self._motion_notify_timeout)
+
     def _click_policy_changed(self):
         self.click_policy = gconf_bridge.get("click_policy")
 
@@ -834,7 +911,11 @@ class Sidebar(gtk.VBox):
         }
 
     def __init__(self, topic):
-        gtk.VBox.__init__(self, False, 4)
+        gtk.VBox.__init__(self, False)
+
+        # FIXME: Not passed to __init__ to avoid binding bug #419107.
+        self.set_spacing(4)
+
         self.topic = topic
         self.in_set_source = False
 
@@ -932,8 +1013,6 @@ class TopicView:
         self.view.show()
         self.scroll.add(self.view)
 
-        self.scroll.connect("size-request", lambda w, req: self._scroll_get_best_size(w, req))
-
     def set_source_by_uri(self, uri):
         for source in self.topic.get_sidebar_source_list():
             if isinstance(source, ItemSource) and source.get_uri() == uri:
@@ -948,26 +1027,8 @@ class TopicView:
     def show_hide_zoomer(self, show):
         pass
 
-    def _scroll_get_best_size(self, scroll, req):
-        vadj = scroll.get_property("vadjustment")
-        upper = vadj.get_property("upper")
-        upper = min(upper, int(self.get_screen().get_height() * 0.7))
-
-        req.height = upper
-        return True
-
-    def _load_items_done(self, ondone_cb):
-        if ondone_cb:
-            ondone_cb()
-
-        vadj = self.scroll.get_property("vadjustment")
-        vadj.set_property("upper", 0)
-
-        w, h = self.child.size_request()
-        self.resize(w, h)
-
     def load_items(self, items, ondone_cb = None):
-        self.view.load_items(items, lambda: self._load_items_done(ondone_cb))
+        self.view.load_items(items, ondone_cb)
 
     def zoom_changed(self, num_days):
         if self.active_source:
@@ -1013,9 +1074,16 @@ class TopicView:
                     self.sidebar.set_source(source)
                     break
 
+                cnt = 0
+                while source.get_items():
+                    cnt += 1
+                    if cnt == min_items:
+                        break
+                else:    
+                    if cnt < min_items:
+                        continue # Skip to the next sidebar button
+
                 items = source.get_items()
-                if len(items) < min_items:
-                    continue # Skip to the next sidebar button
 
                 if source.get_filter_by_date():
                     for zoom in self.get_zoom_level_list():
