@@ -1,7 +1,6 @@
 
 import gc
 import os
-import time
 import urllib
 from gettext import gettext as _
 
@@ -25,22 +24,18 @@ import gimmie_globals
 class FileMonitor(gobject.GObject):
     '''
     A simple wrapper around Gnome VFS file monitors.  Emits created, deleted,
-    and changed events.
+    and changed events.  Incoming events are queued, with the latest event
+    cancelling prior undelivered events.
     '''
     
     __gsignals__ = {
+        "event" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
+                   (gobject.TYPE_STRING, gobject.TYPE_INT)),
         "created" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_STRING,)),
         "deleted" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_STRING,)),
         "changed" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_STRING,))
     }
 
-    event_mapping = {
-        gnomevfs.MONITOR_EVENT_CREATED : "created",
-        gnomevfs.MONITOR_EVENT_DELETED : "deleted",
-        gnomevfs.MONITOR_EVENT_CHANGED : "changed",
-        gnomevfs.MONITOR_EVENT_METADATA_CHANGED : "changed"
-    }
-
     def __init__(self, path):
         gobject.GObject.__init__(self)
 
@@ -50,23 +45,51 @@ class FileMonitor(gobject.GObject):
             self.path = path
         try:
             self.type = gnomevfs.get_file_info(path).type
-        except gnomevfs.NotFoundError:
+        except gnomevfs.Error:
             self.type = gnomevfs.MONITOR_FILE
 
+        self.monitor = None
+        self.pending_timeouts = {}
+
     def open(self):
-        if self.type == gnomevfs.FILE_TYPE_DIRECTORY:
-            monitor_type = gnomevfs.MONITOR_DIRECTORY
-        else:
-            monitor_type = gnomevfs.MONITOR_FILE
-        self.monitor = gnomevfs.monitor_add(self.path, monitor_type, self._callback)
+        if not self.monitor:
+            if self.type == gnomevfs.FILE_TYPE_DIRECTORY:
+                monitor_type = gnomevfs.MONITOR_DIRECTORY
+            else:
+                monitor_type = gnomevfs.MONITOR_FILE
+            self.monitor = gnomevfs.monitor_add(self.path, monitor_type, self._queue_event)
+
+    def _clear_timeout(self, info_uri):
+        try:
+            gobject.source_remove(self.pending_timeouts[info_uri])
+            del self.pending_timeouts[info_uri]
+        except KeyError:
+            pass
+
+    def _queue_event(self, monitor_uri, info_uri, event):
+        self._clear_timeout(info_uri)
+        self.pending_timeouts[info_uri] = \
+            gobject.timeout_add(250, self._timeout_cb, monitor_uri, info_uri, event)
+
+    def queue_changed(self, info_uri):
+        self._queue_event(self.path, info_uri, gnomevfs.MONITOR_EVENT_CHANGED)
 
     def close(self):
         gnomevfs.monitor_cancel(self.monitor)
+        self.monitor = None
 
-    def _callback(self, monitor_uri, info_uri, event):
-        signal = self.event_mapping[event]
-        if signal:
-            self.emit(signal, info_uri)
+    def _timeout_cb(self, monitor_uri, info_uri, event):
+        if event in (gnomevfs.MONITOR_EVENT_METADATA_CHANGED,
+                     gnomevfs.MONITOR_EVENT_CHANGED):
+            self.emit("changed", info_uri)
+        elif event == gnomevfs.MONITOR_EVENT_CREATED:
+            self.emit("created", info_uri)
+        elif event == gnomevfs.MONITOR_EVENT_DELETED:
+            self.emit("deleted", info_uri)
+        self.emit("event", info_uri, event)
+
+        self._clear_timeout(info_uri)
+        return False
 
 
 class FocusRaiser:
@@ -245,6 +268,8 @@ class ToolMenuButton(gtk.ToggleToolButton):
         self.set_label_widget(box)
         self.set_icon_widget(img)
 
+        self.menu = None
+
         self.child.connect("button-press-event", self.button_press)
 
     def set_label(self, label):
@@ -486,6 +511,77 @@ class IconFactory:
         return pixbuf
 
 
+class Thumbnailer:
+    def __init__(self, uri, mimetype):
+        self.uri = uri or ""
+        self.mimetype = mimetype or ""
+        self.cached_icon = None
+        self.cached_timestamp = None
+        self.cached_size = None
+
+    def get_icon(self, icon_size, timestamp = 0):
+        if not self.cached_icon or \
+               icon_size != self.cached_size or \
+               timestamp != self.cached_timestamp:
+            self.cached_icon = self._lookup_or_make_thumb(icon_size, timestamp)
+            self.cached_size = icon_size
+            self.cached_timestamp = timestamp
+        return self.cached_icon
+
+    def _lookup_or_make_thumb(self, icon_size, timestamp):
+        ### FIXME: Trash URIs crash older libgnomeui's icon lookup, so we avoid it
+        if self.uri.startswith("trash:"):
+            return self._get_trash_icon()
+
+        icon_name, icon_type = \
+                   gnome.ui.icon_lookup(icon_theme, thumb_factory, self.uri, self.mimetype, 0)
+        try:
+            if icon_type == gnome.ui.ICON_LOOKUP_RESULT_FLAGS_THUMBNAIL or \
+                   thumb_factory.has_valid_failed_thumbnail(self.uri, timestamp):
+                # Use existing thumbnail
+                thumb = icon_factory.load_icon(icon_name, icon_size)
+            elif self._is_local_uri(self.uri):
+                # Generate a thumbnail for local files only
+                print " *** Calling generate_thumbnail for", self.uri
+                thumb = thumb_factory.generate_thumbnail(self.uri, self.mimetype)
+                thumb_factory.save_thumbnail(thumb, self.uri, timestamp)
+
+            if thumb:
+                # Fixup the thumbnail a bit
+                thumb = self._nicer_dimensions(thumb)
+                thumb = icon_factory.make_icon_frame(thumb, icon_size)
+
+                return thumb
+        except:
+            pass
+
+        # Fallback to mime-type icon on failure
+        return icon_factory.load_icon(icon_name, icon_size)
+
+    def _get_trash_icon(self, uri):
+        trash_icon = "gnome-fs-trash-full"
+        import gimmie_trash
+        if gimmie_trash.trash_monitor.is_empty():
+            trash_icon = "gnome-fs-trash-empty"
+        return icon_factory.load_icon(trash_icon, icon_size)
+
+    def _is_local_uri(self, uri):
+        # NOTE: gnomevfs.URI.is_local seems to hang for some URIs (e.g. ssh
+        #       or http).  So look in a list of local schemes which comes
+        #       directly from gnome_vfs_uri_is_local_scheme.
+        scheme, path = urllib.splittype(self.get_uri() or "")
+        return not scheme or scheme in ("file", "help", "ghelp", "gnome-help", "trash",
+                                        "man", "info", "hardware", "search", "pipe",
+                                        "gnome-trash")
+
+    def _nicer_dimensions(self, icon):
+        ### Constrain thumb dimensions to 1:1.2
+        if float(icon.get_height()) / float(icon.get_width()) > 1.2:
+            return icon.subpixbuf(0, 0,
+                                  icon.get_width(), int(icon.get_width() * 1.2))
+        return icon
+
+
 class LaunchManager:
     '''
     A program lauching utility which handles opening a URI or executing a
@@ -499,16 +595,15 @@ class LaunchManager:
     See the startup notification spec for more information on
     DESKTOP_STARTUP_IDs.
     '''
-    
-    def make_recent_item(self, uri, mimetype):
-        assert uri, "Must specify URI to launch"
-        assert mimetype, "Must specify URI's mimetype"
+    def __init__(self):
+        self.recent_model = None
 
-        recent_item = RecentItem(uri)
-        recent_item.set_mime_type(mimetype)
-        recent_item.set_timestamp(int(time.time()))
-        recent_item.add_group("GimmieWasHere")
-        return recent_item
+    def _get_recent_model(self):
+        # FIXME: This avoids import cycles
+        if not self.recent_model:
+            import gimmie_recent
+            self.recent_model = gimmie_recent.recent_model
+        return self.recent_model
 
     def launch_uri(self, uri, mimetype = None):
         assert uri, "Must specify URI to launch"
@@ -537,8 +632,7 @@ class LaunchManager:
                 except (ImportError, NameError):
                     print " !!! No mimetype found for URI: %s" % uri
 
-            recent_item = self.make_recent_item(uri, mimetype)
-            recent_model.add_full(recent_item)
+            self._get_recent_model().add(uri=uri, mimetype=mimetype)
         return child
 
     def get_local_path(self, uri):
@@ -621,9 +715,9 @@ class LaunchManager:
         else:
             os.wait()
             if launcher_uri:
-                recent_item = self.make_recent_item(launcher_uri, "application/x-desktop")
-                recent_item.add_group("Launchers")
-                recent_model.add_full(recent_item)
+                self._get_recent_model().add(uri=launcher_uri,
+                                            mimetype="application/x-desktop",
+                                            groups=["Launchers"])
             return (child, startup_id)
 
 
@@ -649,7 +743,12 @@ class BookmarkManager(gobject.GObject):
     def __init__(self):
         gobject.GObject.__init__(self)
         self.bookmarks_path = os.path.expanduser("~/.gimmie-bookmarks")
-        self.emit("reload")        
+
+        self.monitor = FileMonitor(self.bookmarks_path)
+        self.monitor.connect("event", lambda a, b, ev: self.emit("reload"))
+        self.monitor.open()
+
+        self.emit("reload")
 
     def do_reload(self):
         self.bookmarks = []
@@ -673,7 +772,9 @@ class BookmarkManager(gobject.GObject):
             f = file(self.bookmarks_path, "w")
             f.writelines(["%s\t%s\t%s\n" % (u, m, c) for u, m, c in self.bookmarks])
             f.close()
-            self.emit("reload")
+
+            # Let the file monitor event signal the change to avoid reloading twice
+            self.monitor.queue_changed(self.bookmarks_path)
         except (IOError, EOFError):
             pass # Doesn't exist, or no access
 
@@ -681,7 +782,7 @@ class BookmarkManager(gobject.GObject):
         assert uri, "Must specify URI to bookmark"
         assert mimetype, "Must specify MimeType for URI"
 
-        if not self.is_bookmark(uri):
+        if not [uri, mimetype, classname] in self.bookmarks:
             self.bookmarks.append([uri, mimetype, classname])
             self._write()
 
@@ -741,10 +842,9 @@ class PlacesManager(gobject.GObject):
         self._bookmarks_path = os.path.expanduser("~/.gtk-bookmarks")
         self._places = []
 
-        monitor = FileMonitor(self._bookmarks_path)
-        monitor.connect("created", lambda a, b: self.emit("reload"))
-        monitor.connect("changed", lambda a, b: self.emit("reload"))
-        monitor.open()
+        self.monitor = FileMonitor(self._bookmarks_path)
+        self.monitor.connect("event", lambda a, b, ev: self.emit("reload"))
+        self.monitor.open()
 
         self.emit("reload")
 
@@ -779,144 +879,6 @@ class PlacesManager(gobject.GObject):
         return self._places
 
 
-#
-# egg.Recent and gtk.RecentManager integration.
-#
-
-if gtk.gtk_version[1] >= 10:
-    ### gtk.RecentManager is only available in >= GTK+ 2.10.
-    
-    class RecentItem:
-        '''
-        Provides a similar interface to egg.recent.RecentItem, for use with 
-        GtkRecent.
-        '''
-        def __init__(self, uri):
-            self.uri = uri
-            self.mimetype = None
-            self.modified = None
-            self.groups = []
-
-        def set_mime_type(self, mimetype):
-            self.mimetype = mimetype
-
-        def set_timestamp(self, modified):
-            self.modified = modified
-
-        def get_timestamp(self):
-            return self.modified
-
-        def add_group(self, group):
-            self.groups.append(group)
-
-        def in_group(self, group):
-            return group in self.groups
-
-        def get_private(self):
-            return False
-
-        def get_uri(self):
-            return self.uri
-
-        def as_dict(self):
-            recent_dict = {'app_name' : 'gimmie',
-                           'app_exec' : 'gimmie'}
-            if self.mimetype:
-                recent_dict['mime_type'] = self.mimetype
-            if self.modified:
-                recent_dict['modified'] = self.modified
-            if self.groups:
-                recent_dict['groups'] = self.groups
-
-            return recent_dict
-
-
-    class CompatRecentItem:
-        def __init__(self, recent_item):
-            self.recent_item = recent_item
-
-        def get_private(self):
-            return self.recent_item.get_private_hint()
-
-        def in_group(self, group):
-            return self.recent_item.has_group(group)
-
-        def get_timestamp(self):
-            return self.recent_item.get_modified()
-
-        def __getattr__(self, x):
-            return getattr(self.recent_item, x)
-
-
-    class RecentlyUsedManager(gtk.RecentManager):
-        def __init__(self):
-            gtk.RecentManager.__init__(self)
-            # FIXME: RecentModel only sends changes if item limit is set
-            self.set_limit(1000000)
-
-            self.recent_items = {}
-            self.connect("changed", self._model_changed)
-            self._model_changed()
-
-        def _model_changed(self, model = None, list = None):
-            self.recent_items = {}
-            for recent_item in self.get_items():
-                recent_item = CompatRecentItem(recent_item)
-                self.recent_items[recent_item.get_uri()] = recent_item
-
-        def add_full(self, item):
-            # FIXME: Work around egg recent not signalling for in-process adds.
-            self.recent_items[item.get_uri()] = item
-            gtk.RecentManager.add_full(self, item.get_uri(), item.as_dict())
-
-        def get_item(self, uri):
-            # Usually, we're given a file path, but maybe not always
-            if uri[0] == '/':
-                uri = 'file://' + uri
-            return self.recent_items[uri]
-
-        def get_list(self):
-            # Backwards compat with egg.recent
-            return self.recent_items.values()
-
-        
-else:
-    ### GTK isn't new enough, go with the libegg code.
-    
-    import egg.recent
-
-
-    class RecentItem(egg.recent.RecentItem):
-        pass
-
-
-    class RecentlyUsedManager(egg.recent.RecentModel):
-        def __init__(self):
-            egg.recent.RecentModel.__init__(self, egg.recent.RECENT_MODEL_SORT_MRU)
-            # FIXME: EggRecentModel only sends changes if item limit is set
-            self.set_limit(1000000)
-
-            self.recent_items = {}
-            self.connect("changed", self._model_changed)
-            self._model_changed()
-
-        def _model_changed(self, model = None, list = None):
-            self.recent_items = {}
-            for recent_item in self.get_list():
-                self.recent_items[recent_item.get_uri()] = recent_item
-
-        def add_full(self, item):
-            # FIXME: Work around egg recent not signalling for in-process adds.
-            self.recent_items[item.get_uri()] = item
-            egg.recent.RecentModel.add_full(self, item)
-
-        def get_item(self, uri):
-            # Usually, we're given a file path, but maybe not always
-            if uri[0] == '/':
-                uri = 'file://' + uri
-            return self.recent_items[uri]
-
-
 class ImplementMe(gtk.MessageDialog):
     '''
     A fun helper that will pop up a "Not Implemented" dialog, and point towards
@@ -1090,9 +1052,10 @@ class DBusWrapper:
                 print " !!! %s D-BUS service not available." % service
 
     def __getattr__(self, name):
-        if not self.obj:
+        try:
+            return getattr(self.obj, name)
+        except AttributeError:
             raise dbus.DBusException
-        return getattr(self.obj, name)
 
 
 class DebugHelper:
@@ -1142,10 +1105,6 @@ icon_theme.append_search_path(gimmie_globals.image_dir)
 thumb_factory = gnome.ui.ThumbnailFactory("normal")
 
 launcher = LaunchManager()
-
-recent_model = RecentlyUsedManager()
-
 bookmarks = BookmarkManager()
 places = PlacesManager()
 gconf_bridge = GConfBridge()
-
